/* File: tables.c */

/* Purpose: Angband Tables */

/*
 * Copyright (c) 1989 James E. Wilson, Robert A. Koeneke
 *
 * This software may be copied and distributed for educational, research, and
 * not for profit purposes provided that this copyright and statement are
 * included in all such copies.
 */

#include "angband.h"




/*
 * Global array for looping through the "keypad directions"
 */
s16b ddd[9] =
{ 2, 8, 6, 4, 3, 1, 9, 7, 5 };

/*
 * Global arrays for converting "keypad direction" into offsets
 */
s16b ddx[10] =
{ 0, -1, 0, 1, -1, 0, 1, -1, 0, 1 };

s16b ddy[10] =
{ 0, 1, 1, 1, 0, 0, 0, -1, -1, -1 };

/*
 * Global arrays for optimizing "ddx[ddd[i]]" and "ddy[ddd[i]]"
 */
s16b ddx_ddd[9] =
{ 0, 0, 1, -1, 1, -1, 1, -1, 0 };

s16b ddy_ddd[9] =
{ 1, -1, 0, 0, 1, 1, -1, -1, 0 };


/*
 * Circular keypad direction array
 */
s16b cdd[8] =
{ 2, 3, 6, 9, 8, 7, 4, 1 };

/*
 * Global arrays for optimizing "ddx[cdd[i]]" and "ddy[cdd[i]]"
 */
s16b ddx_cdd[8] =
{ 0, 1, 1, 1, 0, -1, -1, -1 };

s16b ddy_cdd[8] =
{ 1, 1, 0, -1, -1, -1, 0, 1 };



/*
 * Global array for converting numbers to uppercase hecidecimal digit
 * This array can also be used to convert a number to an octal digit
 */
char hexsym[16] =
{
	'0', '1', '2', '3', '4', '5', '6', '7',
	'8', '9', 'A', 'B', 'C', 'D', 'E', 'F'
};


/*
 * Global array for converting numbers to a logical list symbol
 */
char listsym[] =
{
	'0', '1', '2', '3', '4', '5', '6', '7', '8', '9',
	'a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l', 'm',
	'n', 'o', 'p', 'q', 'r', 's', 't', 'u', 'v', 'w', 'x', 'y', 'z',
	'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M',
	'N', 'O', 'P', 'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X', 'Y', 'Z',
	'\0'
};


/*
 * Encode the screen colors
 */
cptr color_char = "dwsorgbuDWvyRGBU";


/*
 * Stat Table (INT/WIS) -- Number of half-spells per level
 */
byte adj_mag_study[] =
{
	0	/* 3 */,
	0	/* 4 */,
	0	/* 5 */,
	0	/* 6 */,
	0	/* 7 */,
	1	/* 8 */,
	1	/* 9 */,
	1	/* 10 */,
	1	/* 11 */,
	2	/* 12 */,
	2	/* 13 */,
	2	/* 14 */,
	2	/* 15 */,
	2	/* 16 */,
	2	/* 17 */,
	2	/* 18/00-18/09 */,
	2	/* 18/10-18/19 */,
	2	/* 18/20-18/29 */,
	2	/* 18/30-18/39 */,
	2	/* 18/40-18/49 */,
	3	/* 18/50-18/59 */,
	3	/* 18/60-18/69 */,
	3	/* 18/70-18/79 */,
	3	/* 18/80-18/89 */,
	4	/* 18/90-18/99 */,
	4	/* 18/100-18/109 */,
	4	/* 18/110-18/119 */,
	5	/* 18/120-18/129 */,
	5	/* 18/130-18/139 */,
	5	/* 18/140-18/149 */,
	5	/* 18/150-18/159 */,
	5	/* 18/160-18/169 */,
	5	/* 18/170-18/179 */,
	5	/* 18/180-18/189 */,
	5	/* 18/190-18/199 */,
	5	/* 18/200-18/209 */,
	6	/* 18/210-18/219 */,
	6	/* 18/220+ */
};


/*
 * Stat Table (INT/WIS) -- extra 1/4-mana-points per level (plus 128)
 */
byte adj_mag_mana[] =
{
	128 + -8	/* 3 */,
	128 + -6	/* 4 */,
	128 + -4	/* 5 */,
	128 + -2	/* 6 */,
	128 + -1	/* 7 */,
	128 + 0	/* 8 */,
	128 + 0	/* 9 */,
	128 + 0	/* 10 */,
	128 + 0	/* 11 */,
	128 + 0	/* 12 */,
	128 + 0	/* 13 */,
	128 + 1	/* 14 */,
	128 + 1	/* 15 */,
	128 + 2	/* 16 */,
	128 + 3	/* 17 */,
	128 + 4	/* 18/00-18/09 */,
	128 + 5	/* 18/10-18/19 */,
	128 + 6	/* 18/20-18/29 */,
	128 + 7	/* 18/30-18/39 */,
	128 + 8	/* 18/40-18/49 */,
	128 + 9	/* 18/50-18/59 */,
	128 + 10	/* 18/60-18/69 */,
	128 + 11	/* 18/70-18/79 */,
	128 + 12	/* 18/80-18/89 */,
	128 + 13	/* 18/90-18/99 */,
	128 + 14	/* 18/100-18/109 */,
	128 + 15	/* 18/110-18/119 */,
	128 + 16	/* 18/120-18/129 */,
	128 + 17	/* 18/130-18/139 */,
	128 + 18	/* 18/140-18/149 */,
	128 + 19	/* 18/150-18/159 */,
	128 + 20	/* 18/160-18/169 */,
	128 + 21	/* 18/170-18/179 */,
	128 + 22	/* 18/180-18/189 */,
	128 + 23	/* 18/190-18/199 */,
	128 + 24	/* 18/200-18/209 */,
	128 + 25	/* 18/210-18/219 */,
	128 + 26	/* 18/220+ */
};


/*
 * Stat Table (INT/WIS) -- Minimum failure rate (percentage)
 */
byte adj_mag_fail[] =
{
	99      /* 3 */,
	99      /* 4 */,
	99      /* 5 */,
	99      /* 6 */,
	99      /* 7 */,
	50      /* 8 */,
	30      /* 9 */,
	20      /* 10 */,
	15      /* 11 */,
	12      /* 12 */,
	11      /* 13 */,
	10      /* 14 */,
	9       /* 15 */,
	8       /* 16 */,
	7       /* 17 */,
	6       /* 18/00-18/09 */,
	6       /* 18/10-18/19 */,
	5       /* 18/20-18/29 */,
	5       /* 18/30-18/39 */,
	5       /* 18/40-18/49 */,
	4       /* 18/50-18/59 */,
	4       /* 18/60-18/69 */,
	4       /* 18/70-18/79 */,
	4       /* 18/80-18/89 */,
	3       /* 18/90-18/99 */,
	3       /* 18/100-18/109 */,
	2       /* 18/110-18/119 */,
	2       /* 18/120-18/129 */,
	2       /* 18/130-18/139 */,
	2       /* 18/140-18/149 */,
	1       /* 18/150-18/159 */,
	1       /* 18/160-18/169 */,
	1       /* 18/170-18/179 */,
	1       /* 18/180-18/189 */,
	1       /* 18/190-18/199 */,
	0       /* 18/200-18/209 */,
	0       /* 18/210-18/219 */,
	0       /* 18/220+ */
};


/*
 * Stat Table (INT/WIS) -- Various things
 */
byte adj_mag_stat[] =
{
	0       /* 3 */,
	0       /* 4 */,
	0       /* 5 */,
	0       /* 6 */,
	0       /* 7 */,
	1       /* 8 */,
	1       /* 9 */,
	1       /* 10 */,
	1       /* 11 */,
	1       /* 12 */,
	1       /* 13 */,
	1       /* 14 */,
	2       /* 15 */,
	2       /* 16 */,
	2       /* 17 */,
	3       /* 18/00-18/09 */,
	3       /* 18/10-18/19 */,
	3       /* 18/20-18/29 */,
	3       /* 18/30-18/39 */,
	3       /* 18/40-18/49 */,
	4       /* 18/50-18/59 */,
	4       /* 18/60-18/69 */,
	5       /* 18/70-18/79 */,
	6       /* 18/80-18/89 */,
	7       /* 18/90-18/99 */,
	8       /* 18/100-18/109 */,
	9       /* 18/110-18/119 */,
	10      /* 18/120-18/129 */,
	11      /* 18/130-18/139 */,
	12      /* 18/140-18/149 */,
	13      /* 18/150-18/159 */,
	14      /* 18/160-18/169 */,
	15      /* 18/170-18/179 */,
	16      /* 18/180-18/189 */,
	17      /* 18/190-18/199 */,
	18      /* 18/200-18/209 */,
	19      /* 18/210-18/219 */,
	20      /* 18/220+ */
};


/*
 * Stat Table (CHR) -- payment percentages
 */
byte adj_chr_gold[] =
{
	130     /* 3 */,
	125     /* 4 */,
	122     /* 5 */,
	120     /* 6 */,
	118     /* 7 */,
	116     /* 8 */,
	114     /* 9 */,
	112     /* 10 */,
	110     /* 11 */,
	108     /* 12 */,
	106     /* 13 */,
	104     /* 14 */,
	103     /* 15 */,
	102     /* 16 */,
	101     /* 17 */,
	100     /* 18/00-18/09 */,
	99      /* 18/10-18/19 */,
	98      /* 18/20-18/29 */,
	97      /* 18/30-18/39 */,
	96      /* 18/40-18/49 */,
	95      /* 18/50-18/59 */,
	94      /* 18/60-18/69 */,
	93      /* 18/70-18/79 */,
	92      /* 18/80-18/89 */,
	91      /* 18/90-18/99 */,
	90      /* 18/100-18/109 */,
	89      /* 18/110-18/119 */,
	88      /* 18/120-18/129 */,
	87      /* 18/130-18/139 */,
	86      /* 18/140-18/149 */,
	85      /* 18/150-18/159 */,
	84      /* 18/160-18/169 */,
	83      /* 18/170-18/179 */,
	82      /* 18/180-18/189 */,
	81      /* 18/190-18/199 */,
	80      /* 18/200-18/209 */,
	79      /* 18/210-18/219 */,
	78      /* 18/220+ */
};


/*
 * Stat Table (INT) -- Magic devices
 */
byte adj_int_dev[] =
{
	0       /* 3 */,
	0       /* 4 */,
	0       /* 5 */,
	0       /* 6 */,
	0       /* 7 */,
	1       /* 8 */,
	1       /* 9 */,
	1       /* 10 */,
	1       /* 11 */,
	1       /* 12 */,
	1       /* 13 */,
	1       /* 14 */,
	2       /* 15 */,
	2       /* 16 */,
	2       /* 17 */,
	3       /* 18/00-18/09 */,
	3       /* 18/10-18/19 */,
	4       /* 18/20-18/29 */,
	4       /* 18/30-18/39 */,
	5       /* 18/40-18/49 */,
	5       /* 18/50-18/59 */,
	6       /* 18/60-18/69 */,
	6       /* 18/70-18/79 */,
	7       /* 18/80-18/89 */,
	7       /* 18/90-18/99 */,
	8       /* 18/100-18/109 */,
	9       /* 18/110-18/119 */,
	10      /* 18/120-18/129 */,
	11      /* 18/130-18/139 */,
	12      /* 18/140-18/149 */,
	13      /* 18/150-18/159 */,
	14      /* 18/160-18/169 */,
	15      /* 18/170-18/179 */,
	16      /* 18/180-18/189 */,
	17      /* 18/190-18/199 */,
	18      /* 18/200-18/209 */,
	19      /* 18/210-18/219 */,
	20      /* 18/220+ */
};


/*
 * Stat Table (WIS) -- Saving throw
 */
byte adj_wis_sav[] =
{
	0       /* 3 */,
	0       /* 4 */,
	0       /* 5 */,
	0       /* 6 */,
	0       /* 7 */,
	1       /* 8 */,
	1       /* 9 */,
	1       /* 10 */,
	1       /* 11 */,
	1       /* 12 */,
	1       /* 13 */,
	1       /* 14 */,
	2       /* 15 */,
	2       /* 16 */,
	2       /* 17 */,
	3       /* 18/00-18/09 */,
	3       /* 18/10-18/19 */,
	3       /* 18/20-18/29 */,
	3       /* 18/30-18/39 */,
	3       /* 18/40-18/49 */,
	4       /* 18/50-18/59 */,
	4       /* 18/60-18/69 */,
	5       /* 18/70-18/79 */,
	5       /* 18/80-18/89 */,
	6       /* 18/90-18/99 */,
	7       /* 18/100-18/109 */,
	8       /* 18/110-18/119 */,
	9       /* 18/120-18/129 */,
	10      /* 18/130-18/139 */,
	11      /* 18/140-18/149 */,
	12      /* 18/150-18/159 */,
	13      /* 18/160-18/169 */,
	14      /* 18/170-18/179 */,
	15      /* 18/180-18/189 */,
	16      /* 18/190-18/199 */,
	17      /* 18/200-18/209 */,
	18      /* 18/210-18/219 */,
	19      /* 18/220+ */
};


/*
 * Stat Table (DEX) -- disarming
 */
byte adj_dex_dis[] =
{
	0       /* 3 */,
	0       /* 4 */,
	0       /* 5 */,
	0       /* 6 */,
	0       /* 7 */,
	0       /* 8 */,
	0       /* 9 */,
	0       /* 10 */,
	0       /* 11 */,
	0       /* 12 */,
	1       /* 13 */,
	1       /* 14 */,
	1       /* 15 */,
	2       /* 16 */,
	2       /* 17 */,
	4       /* 18/00-18/09 */,
	4       /* 18/10-18/19 */,
	4       /* 18/20-18/29 */,
	4       /* 18/30-18/39 */,
	5       /* 18/40-18/49 */,
	5       /* 18/50-18/59 */,
	5       /* 18/60-18/69 */,
	6       /* 18/70-18/79 */,
	6       /* 18/80-18/89 */,
	7       /* 18/90-18/99 */,
	8       /* 18/100-18/109 */,
	8       /* 18/110-18/119 */,
	8       /* 18/120-18/129 */,
	8       /* 18/130-18/139 */,
	8       /* 18/140-18/149 */,
	9       /* 18/150-18/159 */,
	9       /* 18/160-18/169 */,
	9       /* 18/170-18/179 */,
	9       /* 18/180-18/189 */,
	9       /* 18/190-18/199 */,
	10      /* 18/200-18/209 */,
	10      /* 18/210-18/219 */,
	10      /* 18/220+ */
};


/*
 * Stat Table (INT) -- disarming
 */
byte adj_int_dis[] =
{
	0       /* 3 */,
	0       /* 4 */,
	0       /* 5 */,
	0       /* 6 */,
	0       /* 7 */,
	1       /* 8 */,
	1       /* 9 */,
	1       /* 10 */,
	1       /* 11 */,
	1       /* 12 */,
	1       /* 13 */,
	1       /* 14 */,
	2       /* 15 */,
	2       /* 16 */,
	2       /* 17 */,
	3       /* 18/00-18/09 */,
	3       /* 18/10-18/19 */,
	3       /* 18/20-18/29 */,
	4       /* 18/30-18/39 */,
	4       /* 18/40-18/49 */,
	5       /* 18/50-18/59 */,
	6       /* 18/60-18/69 */,
	7       /* 18/70-18/79 */,
	8       /* 18/80-18/89 */,
	9       /* 18/90-18/99 */,
	10      /* 18/100-18/109 */,
	10      /* 18/110-18/119 */,
	11      /* 18/120-18/129 */,
	12      /* 18/130-18/139 */,
	13      /* 18/140-18/149 */,
	14      /* 18/150-18/159 */,
	15      /* 18/160-18/169 */,
	16      /* 18/170-18/179 */,
	17      /* 18/180-18/189 */,
	18      /* 18/190-18/199 */,
	19      /* 18/200-18/209 */,
	19      /* 18/210-18/219 */,
	20      /* 18/220+ */
};


/*
 * Stat Table (DEX) -- bonus to ac (plus 128)
 */
byte adj_dex_ta[] =
{
	128 + -4    /*  3 */,
	128 + -3    /*  4 */,
	128 + -2    /*  5 */,
	128 + -1    /*  6 */,
	128 + 0     /*  7 */,
	128 + 0     /*  8 */,
	128 + 0     /*  9 */,
	128 + 0     /* 10 */,
	128 + 0     /* 11 */,
	128 + 0     /* 12 */,
	128 + 0     /* 13 */,
	128 + 0     /* 14 */,
	128 + 1     /* 15 */,
	128 + 1     /* 16 */,
	128 + 1     /* 17 */,
	128 + 2     /* 18/00-18/09 */,
	128 + 2     /* 18/10-18/19 */,
	128 + 2     /* 18/20-18/29 */,
	128 + 2     /* 18/30-18/39 */,
	128 + 2     /* 18/40-18/49 */,
	128 + 3     /* 18/50-18/59 */,
	128 + 3     /* 18/60-18/69 */,
	128 + 3     /* 18/70-18/79 */,
	128 + 4     /* 18/80-18/89 */,
	128 + 5     /* 18/90-18/99 */,
	128 + 6     /* 18/100-18/109 */,
	128 + 7     /* 18/110-18/119 */,
	128 + 8     /* 18/120-18/129 */,
	128 + 9     /* 18/130-18/139 */,
	128 + 9     /* 18/140-18/149 */,
	128 + 10    /* 18/150-18/159 */,
	128 + 11    /* 18/160-18/169 */,
	128 + 12    /* 18/170-18/179 */,
	128 + 13    /* 18/180-18/189 */,
	128 + 14    /* 18/190-18/199 */,
	128 + 15    /* 18/200-18/209 */,
	128 + 15    /* 18/210-18/219 */,
	128 + 16    /* 18/220+ */
};


/*
 * Stat Table (STR) -- bonus to dam (plus 128)
 */
byte adj_str_td[] =
{
	128 + -2    /*  3 */,
	128 + -2    /*  4 */,
	128 + -1    /*  5 */,
	128 + -1    /*  6 */,
	128 + 0     /*  7 */,
	128 + 0     /*  8 */,
	128 + 0     /*  9 */,
	128 + 0     /* 10 */,
	128 + 0     /* 11 */,
	128 + 0     /* 12 */,
	128 + 0     /* 13 */,
	128 + 0     /* 14 */,
	128 + 0     /* 15 */,
	128 + 1     /* 16 */,
	128 + 2     /* 17 */,
	128 + 2     /* 18/00-18/09 */,
	128 + 2     /* 18/10-18/19 */,
	128 + 3     /* 18/20-18/29 */,
	128 + 3     /* 18/30-18/39 */,
	128 + 3     /* 18/40-18/49 */,
	128 + 3     /* 18/50-18/59 */,
	128 + 3     /* 18/60-18/69 */,
	128 + 4     /* 18/70-18/79 */,
	128 + 5     /* 18/80-18/89 */,
	128 + 5     /* 18/90-18/99 */,
	128 + 6     /* 18/100-18/109 */,
	128 + 7     /* 18/110-18/119 */,
	128 + 8     /* 18/120-18/129 */,
	128 + 9     /* 18/130-18/139 */,
	128 + 10    /* 18/140-18/149 */,
	128 + 11    /* 18/150-18/159 */,
	128 + 12    /* 18/160-18/169 */,
	128 + 13    /* 18/170-18/179 */,
	128 + 14    /* 18/180-18/189 */,
	128 + 15    /* 18/190-18/199 */,
	128 + 16    /* 18/200-18/209 */,
	128 + 18    /* 18/210-18/219 */,
	128 + 20    /* 18/220+ */
};


/*
 * Stat Table (DEX) -- bonus to hit (plus 128)
 */
byte adj_dex_th[] =
{
	128 + -3        /* 3 */,
	128 + -2        /* 4 */,
	128 + -2        /* 5 */,
	128 + -1        /* 6 */,
	128 + -1        /* 7 */,
	128 + 0 /* 8 */,
	128 + 0 /* 9 */,
	128 + 0 /* 10 */,
	128 + 0 /* 11 */,
	128 + 0 /* 12 */,
	128 + 0 /* 13 */,
	128 + 0 /* 14 */,
	128 + 0 /* 15 */,
	128 + 1 /* 16 */,
	128 + 2 /* 17 */,
	128 + 3 /* 18/00-18/09 */,
	128 + 3 /* 18/10-18/19 */,
	128 + 3 /* 18/20-18/29 */,
	128 + 3 /* 18/30-18/39 */,
	128 + 3 /* 18/40-18/49 */,
	128 + 4 /* 18/50-18/59 */,
	128 + 4 /* 18/60-18/69 */,
	128 + 4 /* 18/70-18/79 */,
	128 + 4 /* 18/80-18/89 */,
	128 + 5 /* 18/90-18/99 */,
	128 + 6 /* 18/100-18/109 */,
	128 + 7 /* 18/110-18/119 */,
	128 + 8 /* 18/120-18/129 */,
	128 + 9 /* 18/130-18/139 */,
	128 + 9 /* 18/140-18/149 */,
	128 + 10        /* 18/150-18/159 */,
	128 + 11        /* 18/160-18/169 */,
	128 + 12        /* 18/170-18/179 */,
	128 + 13        /* 18/180-18/189 */,
	128 + 14        /* 18/190-18/199 */,
	128 + 15        /* 18/200-18/209 */,
	128 + 15        /* 18/210-18/219 */,
	128 + 16        /* 18/220+ */
};


/*
 * Stat Table (STR) -- bonus to hit (plus 128)
 */
byte adj_str_th[] =
{
	128 + -3        /* 3 */,
	128 + -2        /* 4 */,
	128 + -1        /* 5 */,
	128 + -1        /* 6 */,
	128 + 0 /* 7 */,
	128 + 0 /* 8 */,
	128 + 0 /* 9 */,
	128 + 0 /* 10 */,
	128 + 0 /* 11 */,
	128 + 0 /* 12 */,
	128 + 0 /* 13 */,
	128 + 0 /* 14 */,
	128 + 0 /* 15 */,
	128 + 0 /* 16 */,
	128 + 0 /* 17 */,
	128 + 1 /* 18/00-18/09 */,
	128 + 1 /* 18/10-18/19 */,
	128 + 1 /* 18/20-18/29 */,
	128 + 1 /* 18/30-18/39 */,
	128 + 1 /* 18/40-18/49 */,
	128 + 1 /* 18/50-18/59 */,
	128 + 1 /* 18/60-18/69 */,
	128 + 2 /* 18/70-18/79 */,
	128 + 3 /* 18/80-18/89 */,
	128 + 4 /* 18/90-18/99 */,
	128 + 5 /* 18/100-18/109 */,
	128 + 6 /* 18/110-18/119 */,
	128 + 7 /* 18/120-18/129 */,
	128 + 8 /* 18/130-18/139 */,
	128 + 9 /* 18/140-18/149 */,
	128 + 10        /* 18/150-18/159 */,
	128 + 11        /* 18/160-18/169 */,
	128 + 12        /* 18/170-18/179 */,
	128 + 13        /* 18/180-18/189 */,
	128 + 14        /* 18/190-18/199 */,
	128 + 15        /* 18/200-18/209 */,
	128 + 15        /* 18/210-18/219 */,
	128 + 16        /* 18/220+ */
};


/*
 * Stat Table (STR) -- weight limit in deca-pounds
 */
byte adj_str_wgt[] =
{
	10      /* 3 */,
	11      /* 4 */,
	12      /* 5 */,
	13      /* 6 */,
	14      /* 7 */,
	15      /* 8 */,
	16      /* 9 */,
	17      /* 10 */,
	18      /* 11 */,
	19      /* 12 */,
	20      /* 13 */,
	21      /* 14 */,
	22      /* 15 */,
	23      /* 16 */,
	24      /* 17 */,
	25      /* 18/00-18/09 */,
	26      /* 18/10-18/19 */,
	27      /* 18/20-18/29 */,
	28      /* 18/30-18/39 */,
	29      /* 18/40-18/49 */,
	30      /* 18/50-18/59 */,
	31      /* 18/60-18/69 */,
	31      /* 18/70-18/79 */,
	32      /* 18/80-18/89 */,
	32      /* 18/90-18/99 */,
	33      /* 18/100-18/109 */,
	33      /* 18/110-18/119 */,
	34      /* 18/120-18/129 */,
	34      /* 18/130-18/139 */,
	35      /* 18/140-18/149 */,
	35      /* 18/150-18/159 */,
	36      /* 18/160-18/169 */,
	36      /* 18/170-18/179 */,
	37      /* 18/180-18/189 */,
	37      /* 18/190-18/199 */,
	38      /* 18/200-18/209 */,
	38      /* 18/210-18/219 */,
	39      /* 18/220+ */
};


/*
 * Stat Table (STR) -- weapon weight limit in pounds
 */
byte adj_str_hold[] =
{
	4       /* 3 */,
	5       /* 4 */,
	6       /* 5 */,
	7       /* 6 */,
	8       /* 7 */,
	9       /* 8 */,
	10      /* 9 */,
	11      /* 10 */,
	12      /* 11 */,
	13      /* 12 */,
	14      /* 13 */,
	15      /* 14 */,
	16      /* 15 */,
	17      /* 16 */,
	18      /* 17 */,
	19      /* 18/00-18/09 */,
	20      /* 18/10-18/19 */,
	21      /* 18/20-18/29 */,
	22      /* 18/30-18/39 */,
	23      /* 18/40-18/49 */,
	24      /* 18/50-18/59 */,
	25      /* 18/60-18/69 */,
	26      /* 18/70-18/79 */,
	27      /* 18/80-18/89 */,
	28      /* 18/90-18/99 */,
	30      /* 18/100-18/109 */,
	31      /* 18/110-18/119 */,
	32      /* 18/120-18/129 */,
	33      /* 18/130-18/139 */,
	34      /* 18/140-18/149 */,
	35      /* 18/150-18/159 */,
	37      /* 18/160-18/169 */,
	40      /* 18/170-18/179 */,
	44      /* 18/180-18/189 */,
	48      /* 18/190-18/199 */,
	50     /* 18/200-18/209 */,
	50     /* 18/210-18/219 */,
	50     /* 18/220+ */
};


/*
 * Stat Table (STR) -- digging value
 */
byte adj_str_dig[] =
{
	0       /* 3 */,
	0       /* 4 */,
	1       /* 5 */,
	2       /* 6 */,
	3       /* 7 */,
	4       /* 8 */,
	4       /* 9 */,
	5       /* 10 */,
	5       /* 11 */,
	6       /* 12 */,
	6       /* 13 */,
	7       /* 14 */,
	7       /* 15 */,
	8       /* 16 */,
	8       /* 17 */,
	9       /* 18/00-18/09 */,
	10      /* 18/10-18/19 */,
	12      /* 18/20-18/29 */,
	15      /* 18/30-18/39 */,
	20      /* 18/40-18/49 */,
	25      /* 18/50-18/59 */,
	30      /* 18/60-18/69 */,
	35      /* 18/70-18/79 */,
	40      /* 18/80-18/89 */,
	45      /* 18/90-18/99 */,
	50      /* 18/100-18/109 */,
	55      /* 18/110-18/119 */,
	60      /* 18/120-18/129 */,
	65      /* 18/130-18/139 */,
	70      /* 18/140-18/149 */,
	75      /* 18/150-18/159 */,
	80      /* 18/160-18/169 */,
	85      /* 18/170-18/179 */,
	90      /* 18/180-18/189 */,
	95      /* 18/190-18/199 */,
	100     /* 18/200-18/209 */,
	100     /* 18/210-18/219 */,
	100     /* 18/220+ */
};


/*
 * Stat Table (STR) -- help index into the "blow" table
 */
byte adj_str_blow[] =
{
	3       /* 3 */,
	4       /* 4 */,
	5       /* 5 */,
	6       /* 6 */,
	7       /* 7 */,
	8       /* 8 */,
	9       /* 9 */,
	10      /* 10 */,
	11      /* 11 */,
	12      /* 12 */,
	13      /* 13 */,
	14      /* 14 */,
	15      /* 15 */,
	16      /* 16 */,
	17      /* 17 */,
	20 /* 18/00-18/09 */,
	30 /* 18/10-18/19 */,
	40 /* 18/20-18/29 */,
	50 /* 18/30-18/39 */,
	60 /* 18/40-18/49 */,
	70 /* 18/50-18/59 */,
	80 /* 18/60-18/69 */,
	90 /* 18/70-18/79 */,
	100 /* 18/80-18/89 */,
	110 /* 18/90-18/99 */,
	120 /* 18/100-18/109 */,
	130 /* 18/110-18/119 */,
	140 /* 18/120-18/129 */,
	150 /* 18/130-18/139 */,
	160 /* 18/140-18/149 */,
	170 /* 18/150-18/159 */,
	180 /* 18/160-18/169 */,
	190 /* 18/170-18/179 */,
	200 /* 18/180-18/189 */,
	210 /* 18/190-18/199 */,
	220 /* 18/200-18/209 */,
	230 /* 18/210-18/219 */,
	240 /* 18/220+ */
};


/*
 * Stat Table (DEX) -- index into the "blow" table
 */
byte adj_dex_blow[] =
{
	0       /* 3 */,
	0       /* 4 */,
	0       /* 5 */,
	0       /* 6 */,
	0       /* 7 */,
	0       /* 8 */,
	0       /* 9 */,
	1       /* 10 */,
	1       /* 11 */,
	1       /* 12 */,
	2       /* 13 */,
	2       /* 14 */,
	3       /* 15 */,
	3       /* 16 */,
	3       /* 17 */,
	4       /* 18/00-18/09 */,
	4       /* 18/10-18/19 */,
	5       /* 18/20-18/29 */,
	5       /* 18/30-18/39 */,
	6       /* 18/40-18/49 */,
	6       /* 18/50-18/59 */,
	7       /* 18/60-18/69 */,
	7       /* 18/70-18/79 */,
	8       /* 18/80-18/89 */,
	8       /* 18/90-18/99 */,
	9       /* 18/100-18/109 */,
	9       /* 18/110-18/119 */,
	10       /* 18/120-18/129 */,
	10      /* 18/130-18/139 */,
	10     /* 18/140-18/149 */,
	11      /* 18/150-18/159 */,
	11      /* 18/160-18/169 */,
	11      /* 18/170-18/179 */,
	12      /* 18/180-18/189 */,
	12      /* 18/190-18/199 */,
	12      /* 18/200-18/209 */,
	13      /* 18/210-18/219 */,
	13      /* 18/220+ */
};


/*
 * Stat Table (DEX) -- index into the "blow" table (bare hand)
 */
byte adj_dex_blow_bare_hand[] =
{
	0       /* 3 */,
	0       /* 4 */,
	0       /* 5 */,
	0       /* 6 */,
	0       /* 7 */,
	0       /* 8 */,
	0       /* 9 */,
	1       /* 10 */,
	1       /* 11 */,
	1       /* 12 */,
	1       /* 13 */,
	1       /* 14 */,
	2       /* 15 */,
	2       /* 16 */,
	2       /* 17 */,
	2       /* 18/00-18/09 */,
	3       /* 18/10-18/19 */,
	3       /* 18/20-18/29 */,
	3       /* 18/30-18/39 */,
	4       /* 18/40-18/49 */,
	4       /* 18/50-18/59 */,
	6       /* 18/60-18/69 */,
	6       /* 18/70-18/79 */,
	8       /* 18/80-18/89 */,
	8       /* 18/90-18/99 */,
	10      /* 18/100-18/109 */,
	10      /* 18/110-18/119 */,
	14      /* 18/120-18/129 */,
	14      /* 18/130-18/139 */,
	18     /* 18/140-18/149 */,
	18     /* 18/150-18/159 */,
	24      /* 18/160-18/169 */,
	24      /* 18/170-18/179 */,
	30      /* 18/180-18/189 */,
	30      /* 18/190-18/199 */,
	36      /* 18/200-18/209 */,
	36      /* 18/210-18/219 */,
	42      /* 18/220+ */
};


/*
 * Stat Table (DEX) -- chance of avoiding "theft" and "falling"
 */
byte adj_dex_safe[] =
{
	0       /* 3 */,
	1       /* 4 */,
	2       /* 5 */,
	3       /* 6 */,
	4       /* 7 */,
	5       /* 8 */,
	5       /* 9 */,
	6       /* 10 */,
	6       /* 11 */,
	7       /* 12 */,
	7       /* 13 */,
	8       /* 14 */,
	8       /* 15 */,
	9       /* 16 */,
	9       /* 17 */,
	10      /* 18/00-18/09 */,
	10      /* 18/10-18/19 */,
	15      /* 18/20-18/29 */,
	15      /* 18/30-18/39 */,
	20      /* 18/40-18/49 */,
	25      /* 18/50-18/59 */,
	30      /* 18/60-18/69 */,
	35      /* 18/70-18/79 */,
	40      /* 18/80-18/89 */,
	45      /* 18/90-18/99 */,
	50      /* 18/100-18/109 */,
	60      /* 18/110-18/119 */,
	70      /* 18/120-18/129 */,
	80      /* 18/130-18/139 */,
	90      /* 18/140-18/149 */,
	100     /* 18/150-18/159 */,
	100     /* 18/160-18/169 */,
	100     /* 18/170-18/179 */,
	100     /* 18/180-18/189 */,
	100     /* 18/190-18/199 */,
	100     /* 18/200-18/209 */,
	100     /* 18/210-18/219 */,
	100     /* 18/220+ */
};


/*
 * Stat Table (CON) -- base regeneration rate
 */
byte adj_con_fix[] =
{
	0       /* 3 */,
	0       /* 4 */,
	0       /* 5 */,
	0       /* 6 */,
	0       /* 7 */,
	0       /* 8 */,
	0       /* 9 */,
	0       /* 10 */,
	0       /* 11 */,
	0       /* 12 */,
	0       /* 13 */,
	1       /* 14 */,
	1       /* 15 */,
	1       /* 16 */,
	1       /* 17 */,
	2       /* 18/00-18/09 */,
	2       /* 18/10-18/19 */,
	2       /* 18/20-18/29 */,
	2       /* 18/30-18/39 */,
	2       /* 18/40-18/49 */,
	3       /* 18/50-18/59 */,
	3       /* 18/60-18/69 */,
	3       /* 18/70-18/79 */,
	3       /* 18/80-18/89 */,
	3       /* 18/90-18/99 */,
	4       /* 18/100-18/109 */,
	4       /* 18/110-18/119 */,
	5       /* 18/120-18/129 */,
	6       /* 18/130-18/139 */,
	6       /* 18/140-18/149 */,
	7       /* 18/150-18/159 */,
	7       /* 18/160-18/169 */,
	8       /* 18/170-18/179 */,
	8       /* 18/180-18/189 */,
	8       /* 18/190-18/199 */,
	9       /* 18/200-18/209 */,
	9       /* 18/210-18/219 */,
	9       /* 18/220+ */
};


/*
 * Stat Table (CON) -- extra 1/4-hitpoints per level (plus 128)
 */
byte adj_con_mhp[] =
{
	128 + -8	/* 3 */,
	128 + -6	/* 4 */,
	128 + -4	/* 5 */,
	128 + -2	/* 6 */,
	128 + -1	/* 7 */,
	128 + 0	/* 8 */,
	128 + 0	/* 9 */,
	128 + 0	/* 10 */,
	128 + 0	/* 11 */,
	128 + 0	/* 12 */,
	128 + 0	/* 13 */,
	128 + 1	/* 14 */,
	128 + 1	/* 15 */,
	128 + 2	/* 16 */,
	128 + 3	/* 17 */,
	128 + 4	/* 18/00-18/09 */,
	128 + 5	/* 18/10-18/19 */,
	128 + 6	/* 18/20-18/29 */,
	128 + 7	/* 18/30-18/39 */,
	128 + 8	/* 18/40-18/49 */,
	128 + 9	/* 18/50-18/59 */,
	128 + 10	/* 18/60-18/69 */,
	128 + 11	/* 18/70-18/79 */,
	128 + 12	/* 18/80-18/89 */,
	128 + 13	/* 18/90-18/99 */,
	128 + 14	/* 18/100-18/109 */,
	128 + 15	/* 18/110-18/119 */,
	128 + 16	/* 18/120-18/129 */,
	128 + 17	/* 18/130-18/139 */,
	128 + 18	/* 18/140-18/149 */,
	128 + 19	/* 18/150-18/159 */,
	128 + 20	/* 18/160-18/169 */,
	128 + 21	/* 18/170-18/179 */,
	128 + 22	/* 18/180-18/189 */,
	128 + 23	/* 18/190-18/199 */,
	128 + 24	/* 18/200-18/209 */,
	128 + 25	/* 18/210-18/219 */,
	128 + 26	/* 18/220+ */
};


/*
 * Stat Table (CHR) -- charm
 */
byte adj_chr_chm[] =
{
	0       /* 3 */,
	0       /* 4 */,
	1       /* 5 */,
	2       /* 6 */,
	3       /* 7 */,
	4       /* 8 */,
	4       /* 9 */,
	5       /* 10 */,
	5       /* 11 */,
	6       /* 12 */,
	6       /* 13 */,
	7       /* 14 */,
	7       /* 15 */,
	8       /* 16 */,
	8       /* 17 */,
	9       /* 18/00-18/09 */,
	10      /* 18/10-18/19 */,
	12      /* 18/20-18/29 */,
	15      /* 18/30-18/39 */,
	18      /* 18/40-18/49 */,
	21      /* 18/50-18/59 */,
	24      /* 18/60-18/69 */,
	28      /* 18/70-18/79 */,
	32      /* 18/80-18/89 */,
	36      /* 18/90-18/99 */,
	39      /* 18/100-18/109 */,
	42      /* 18/110-18/119 */,
	45      /* 18/120-18/129 */,
	49      /* 18/130-18/139 */,
	53      /* 18/140-18/149 */,
	57      /* 18/150-18/159 */,
	61      /* 18/160-18/169 */,
	65      /* 18/170-18/179 */,
	69      /* 18/180-18/189 */,
	73      /* 18/190-18/199 */,
	77      /* 18/200-18/209 */,
	81      /* 18/210-18/219 */,
	85      /* 18/220+ */
};


/*
 * Stat Table (CHR) -- give money
 */
byte adj_chr_give[] =
{
	128 + -5      /* 3 */,
	128 + -4      /* 4 */,
	128 + -3      /* 5 */,
	128 + -2      /* 6 */,
	128 + -1      /* 7 */,
	128 + 0       /* 8 */,
	128 + 0       /* 9 */,
	128 + 0       /* 10 */,
	128 + 0       /* 11 */,
	128 + 0       /* 12 */,
	128 + 0       /* 13 */,
	128 + 1       /* 14 */,
	128 + 1       /* 15 */,
	128 + 2       /* 16 */,
	128 + 3       /* 17 */,
	128 + 4       /* 18/00-18/09 */,
	128 + 5       /* 18/10-18/19 */,
	128 + 6       /* 18/20-18/29 */,
	128 + 7       /* 18/30-18/39 */,
	128 + 8       /* 18/40-18/49 */,
	128 + 9       /* 18/50-18/59 */,
	128 + 10      /* 18/60-18/69 */,
	128 + 11      /* 18/70-18/79 */,
	128 + 12      /* 18/80-18/89 */,
	128 + 13      /* 18/90-18/99 */,
	128 + 14      /* 18/100-18/109 */,
	128 + 15      /* 18/110-18/119 */,
	128 + 16      /* 18/120-18/129 */,
	128 + 17      /* 18/130-18/139 */,
	128 + 18      /* 18/140-18/149 */,
	128 + 19      /* 18/150-18/159 */,
	128 + 20      /* 18/160-18/169 */,
	128 + 21      /* 18/170-18/179 */,
	128 + 22      /* 18/180-18/189 */,
	128 + 23      /* 18/190-18/199 */,
	128 + 24      /* 18/200-18/209 */,
	128 + 25      /* 18/210-18/219 */,
	128 + 26      /* 18/220+ */
};


/*
 * Stat Table (STR/DEX) -- throwing stone or grave damages and to-hit (plus 128)
 */
byte adj_throwing[] =
{
	128 + 0     /*  3 */,
	128 + 0     /*  4 */,
	128 + 0     /*  5 */,
	128 + 0     /*  6 */,
	128 + 0     /*  7 */,
	128 + 0     /*  8 */,
	128 + 0     /*  9 */,
	128 + 0     /* 10 */,
	128 + 0     /* 11 */,
	128 + 0     /* 12 */,
	128 + 0     /* 13 */,
	128 + 0     /* 14 */,
	128 + 0     /* 15 */,
	128 + 0     /* 16 */,
	128 + 0     /* 17 */,
	128 + 1     /* 18/00-18/09 */,
	128 + 1     /* 18/10-18/19 */,
	128 + 1     /* 18/20-18/29 */,
	128 + 1     /* 18/30-18/39 */,
	128 + 1     /* 18/40-18/49 */,
	128 + 2     /* 18/50-18/59 */,
	128 + 2     /* 18/60-18/69 */,
	128 + 2     /* 18/70-18/79 */,
	128 + 2     /* 18/80-18/89 */,
	128 + 2     /* 18/90-18/99 */,
	128 + 3     /* 18/100-18/109 */,
	128 + 3     /* 18/110-18/119 */,
	128 + 3     /* 18/120-18/129 */,
	128 + 3     /* 18/130-18/139 */,
	128 + 3     /* 18/140-18/149 */,
	128 + 4     /* 18/150-18/159 */,
	128 + 4     /* 18/160-18/169 */,
	128 + 4     /* 18/170-18/179 */,
	128 + 4     /* 18/180-18/189 */,
	128 + 4     /* 18/190-18/199 */,
	128 + 5     /* 18/200-18/209 */,
	128 + 5     /* 18/210-18/219 */,
	128 + 5     /* 18/220+ */
};


/*
 * This table is used to help calculate the number of blows the player can
 * make in a single round of attacks (one player turn) with a normal weapon.
 *
 * This number ranges from a single blow/round for weak players to up to six
 * blows/round for powerful warriors.
 *
 * Note that certain artifacts and ego-items give "bonus" blows/round.
 *
 * First, from the player class, we extract some values:
 *
 * Soldier       num = 5; mul = 3; div = MAX(70, weapon_weight)
 * Knight        num = 6; mul = 5; div = MAX(70, weapon_weight)
 * Berserker     num = 6; mul = 5; div = MAX(70, weapon_weight)
 * Terror-Knight num = 6; mul = 6; div = MAX(70, weapon_weight)
 * BeastTamer    num = 5; mul = 4; div = MAX(70, weapon_weight)
 * Sword-Master  num = 5; mul = 4; div = MAX(50, weapon_weight)
 * Dragoon       num = 6; mul = 5; div = MAX(70, weapon_weight)
 * Ninja         num = 4; mul = 1; div = MAX(20, weapon_weight)
 * Wizard        num = 3; mul = 2; div = MAX(100, weapon_weight)
 * Warlock       num = 4; mul = 4; div = MAX(100, weapon_weight)
 * Exorcist      num = 5; mul = 4; div = MAX(100, weapon_weight)
 * Amazoness     num = 5; mul = 3; div = MAX(70, weapon_weight)
 * Valkyrie      num = 5; mul = 4; div = MAX(70, weapon_weight)
 * Archer        num = 4; mul = 3; div = MAX(100, weapon_weight)
 * DragonTamer   num = 5; mul = 4; div = MAX(70, weapon_weight)
 * Witch         num = 4; mul = 3; div = MAX(100, weapon_weight)
 * Sirene        num = 3; mul = 2; div = MAX(100, weapon_weight)
 * Cleric        num = 5; mul = 3; div = MAX(100, weapon_weight)
 * Priest        num = 3; mul = 2; div = MAX(100, weapon_weight)
 * Lich          num = 3; mul = 2; div = MAX(100, weapon_weight)
 * Angel-Knight  num = 6; mul = 5; div = MAX(70, weapon_weight)
 * High-Witch    num = 1; mul = 1; div = MAX(1, weapon_weight)
 * Gunner        num = 3; mul = 2; div = MAX(100, weapon_weight)
 * Temple-Knight num = 5; mul = 4; div = MAX(70, weapon_weight)
 * White-Knight num = 5; mul = 4; div = MAX(70, weapon_weight)
 *
 * To get "P", we look up the relevant "adj_str_blow[]" (see above),
 * multiply it by "mul", and then divide it by "div".
 * Increase P by 1 if you wield a weapon two-handed.
 * Decrease P by 1 if you are a Ninja.
 *
 * To get "D", we look up the relevant "adj_dex_blow[]" (see above),
 *
 * The player gets "blows_table[P][D]" blows/round, as shown below,
 * up to a maximum of "num" blows/round, plus any "bonus" blows/round.
 */
byte blows_table[12][12] =
{
	/* P/D */
	/*      0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11+ */
	/*      3   10   15  /10  /40  /60  /80 /100 /120 /140 /160 /180  */
/* 0 */{	1,   1,   1,   2,   2,   2,   2,   2,   2,   3,   3,   4 },
/* 1 */{	1,   1,   2,   2,   2,   2,   3,   3,   3,   4,   4,   4 },
/* 2 */{	2,   2,   2,   2,   3,   3,   4,   4,   4,   5,   5,   5 },
/* 3 */{	2,   2,   2,   3,   3,   4,   4,   4,   5,   5,   5,   5 },
/* 4 */{	2,   2,   2,   3,   3,   4,   4,   5,   5,   5,   5,   5 },
/* 5 */{	2,   2,   2,   3,   4,   4,   4,   5,   5,   5,   5,   6 },
/* 6 */{	2,   2,   2,   3,   4,   4,   4,   5,   5,   5,   5,   6 },
/* 7 */{	2,   2,   2,   3,   4,   4,   4,   5,   5,   5,   5,   6 },
/* 8 */{	2,   2,   3,   3,   4,   4,   4,   5,   5,   5,   6,   6 },
/* 9 */{	2,   2,   3,   4,   4,   4,   5,   5,   5,   5,   6,   6 },
/* 10*/{	2,   2,   3,   4,   4,   4,   5,   5,   5,   6,   6,   6 },
/*11+*/{	2,   2,   3,   4,   4,   4,   5,   5,   6,   6,   6,   7 },

};

arena_type arena_info[MAX_ARENA_MONS + 4] =
{
	{ MON_SIMON,            TV_BOOTS,  SV_PAIR_OF_SANDALS            },
	{ MON_SOLDIER,          TV_POTION, SV_POTION_CURE_CRITICAL       },
	{ MON_AMAZONESS,        TV_POTION, SV_POTION_CURE_CRITICAL       },
	{ MON_ARCHER,           TV_POTION, SV_POTION_SPEED               },
	{ MON_KNIGHT,           TV_POTION, SV_POTION_HEROISM             },
	{ MON_PRIEST,           TV_POTION, SV_POTION_CURING              },
	{ MON_BEASTTAMER,       TV_TRUMP,  SV_MONSTER_CARD               },
	{ MON_BERSERKER,        TV_POTION, SV_POTION_HEROISM             },
	{ MON_FAIRY,            TV_POTION, SV_POTION_CURING              },
	{ MON_HAWKMAN,          TV_POTION, SV_POTION_HEALING             },
	{ MON_DRAGOON,          TV_POTION, SV_POTION_HEROISM             },
	{ MON_SWORDMASTER,      TV_POTION, SV_POTION_SPEED               },
	{ MON_TERROR_KNIGHT,    TV_POTION, SV_POTION_BERSERK_STRENGTH    },
	{ MON_PIXY,             TV_POTION, SV_POTION_SPEED               },
	{ MON_WARLOCK,          TV_POTION, SV_POTION_RESTORING           },
	{ MON_NINJA,            TV_POTION, SV_POTION_SPEED               },
	{ MON_BEASTMASTER,      TV_ROD,    SV_ROD_STONE_TO_MUD           },
	{ MON_PUMPKIN_HEAD,     TV_ROD,    SV_ROD_MAPPING                },
	{ MON_LIZARDMAN,        TV_SCROLL, SV_SCROLL_STAR_DESTRUCTION    },
	{ MON_GORGON,           TV_POTION, SV_POTION_RESTORE_MANA        },
	{ MON_GUNNER,           TV_BOW,    SV_ROCKET_LAUNCHER            },
	{ MON_COCKATRICE,       TV_POTION, SV_POTION_CURE_STONING        },
	{ MON_VALKYRIE,         TV_POTION, SV_POTION_STAR_HEALING        },
	{ MON_SIRENE,           TV_SCROLL, SV_SCROLL_ACQUIREMENT         },
	{ MON_DRAGONTAMER,      TV_TRUMP,  SV_MONSTER_CARD               },
	{ MON_DARK_PRIEST,      TV_POTION, SV_POTION_RESTORE_MANA        },
	{ MON_SORCERER,         TV_SCROLL, SV_SCROLL_GENOCIDE            },
	{ MON_VENEFIK,          TV_POTION, SV_POTION_DEATH               },
	{ MON_CENTAUR,          TV_RING,   SV_RING_SHOTS                 },
	{ MON_LADY_CENTAUR,     TV_RING,   SV_RING_DEC_MANA              },
	{ MON_HIGH_PRIEST,      TV_POTION, SV_POTION_LIFE                },
	{ MON_NECROMANCER,      TV_SCROLL, SV_SCROLL_MASS_GENOCIDE       },
	{ MON_MAD_HALLOWEEN,    TV_SCROLL, SV_SCROLL_STAR_ACQUIREMENT    },
	{ MON_GABIAR,           TV_ROD,    SV_ROD_HEALING                },
	{ MON_ANGEL_KNIGHT,     TV_ROD,    SV_ROD_SPEED                  },
	{ MON_BALANCE_WYRM,     TV_ROD,    SV_ROD_IDENTIFY               },
	{ MON_FIRE_BRASS,       TV_WAND,   SV_WAND_DRAGON_FIRE           },
	{ MON_MOLDOUX,          TV_STAFF,  SV_STAFF_THE_MAGI             },
	{ 0,                    0,         0                             },
	{ MON_CANOPUS,          0,         0                             },
	{ 0,                    0,         0                             },
	{ MON_DENEB,            0,         0                             },
};



/*
 * This table allows quick conversion from "speed" to "energy"
 * The basic function WAS ((S>=110) ? (S-110) : (100 / (120-S)))
 * Note that table access is *much* quicker than computation.
 *
 * Note that the table has been changed at high speeds.  From
 * "Slow (-40)" to "Fast (+30)" is pretty much unchanged, but
 * at speeds above "Fast (+30)", one approaches an asymptotic
 * effective limit of 50 energy per turn.  This means that it
 * is relatively easy to reach "Fast (+30)" and get about 40
 * energy per turn, but then speed becomes very "expensive",
 * and you must get all the way to "Fast (+50)" to reach the
 * point of getting 45 energy per turn.  After that point,
 * furthur increases in speed are more or less pointless,
 * except to balance out heavy inventory.
 *
 * Note that currently the fastest monster is "Fast (+30)".
 *
 * It should be possible to lower the energy threshhold from
 * 100 units to 50 units, though this may interact badly with
 * the (compiled out) small random energy boost code.  It may
 * also tend to cause more "clumping" at high speeds.
 */
byte extract_energy[200] =
{
	/* Slow */     1,  1,  1,  1,  1,  1,  1,  1,  1,  1,
	/* Slow */     1,  1,  1,  1,  1,  1,  1,  1,  1,  1,
	/* Slow */     1,  1,  1,  1,  1,  1,  1,  1,  1,  1,
	/* Slow */     1,  1,  1,  1,  1,  1,  1,  1,  1,  1,
	/* Slow */     1,  1,  1,  1,  1,  1,  1,  1,  1,  1,
	/* Slow */     1,  1,  1,  1,  1,  1,  1,  1,  1,  1,
	/* S-50 */     1,  1,  1,  1,  1,  1,  1,  1,  1,  1,
	/* S-40 */     2,  2,  2,  2,  2,  2,  2,  2,  2,  2,
	/* S-30 */     2,  2,  2,  2,  2,  2,  2,  3,  3,  3,
	/* S-20 */     3,  3,  3,  3,  3,  4,  4,  4,  4,  4,
	/* S-10 */     5,  5,  5,  5,  6,  6,  7,  7,  8,  9,
	/* Norm */    10, 11, 12, 13, 14, 15, 16, 17, 18, 19,
	/* F+10 */    20, 21, 22, 23, 24, 25, 26, 27, 28, 29,
	/* F+20 */    30, 31, 32, 33, 34, 35, 36, 36, 37, 37,
	/* F+30 */    38, 38, 39, 39, 40, 40, 40, 41, 41, 41,
	/* F+40 */    42, 42, 42, 43, 43, 43, 44, 44, 44, 44,
	/* F+50 */    45, 45, 45, 45, 45, 46, 46, 46, 46, 46,
	/* F+60 */    47, 47, 47, 47, 47, 48, 48, 48, 48, 48,
	/* F+70 */    49, 49, 49, 49, 49, 49, 49, 49, 49, 49,
	/* Fast */    49, 49, 49, 49, 49, 49, 49, 49, 49, 49,
};




/*
 * Base experience levels
 */
s32b player_exp[PY_MAX_LEVEL] =
{
	12,
	31,
	56,
	87,
	125,
	175,
	250,
	350,
	475, /* 10 */
	625,
	812,
	1062,
	1375,
	1750,
	2250,
	2875,
	3625,
	4500,
	5500, /* 20 */
	6750,
	8500,
	10500,
	12750,
	15625,
	21875,
	31250,
	43750L,
	62500L,
	93750L, /* 30 */
	125000L,
	187500L,
	250000L,
	343750L,
	437500L,
	562500L,
	687500L,
	875000L,
	1062500L,
	1250000L, /* 40 */
	1562500L,
	1875000L,
	2250000L,
	2625000L,
	3000000L,
	3375000L,
	3750000L,
	4375000L,
	5000000L,
	5625000L, /* 50 */
	6250000L
};


/*
 * Player Sexes
 *
 *      Title,
 *      Winner
 */
player_sex sex_info[MAX_SEXES] =
{
	{
#ifdef JP
		"",
		"",
#endif
		"Female",
		"Queen"
	},
	{
#ifdef JP
		"",
		"",
#endif
		"Male",
		"King"
	}
};



/*
 * Zangband uses this array instead of the spell flags table, as there
 * are 5 realms of magic, each with 4 spellbooks and 8 spells per book -- TY
 */
u32b fake_spell_flags[MAX_REALM][3] =
{
	/* Magery */
	{
		0x00001fff,
		0x03ffe000,
		0x00000000
	},
	/* Fire Magic */
	{
		0x00003fff,
		0x00000000,
		0x00000000
	},
	/* Aqua Magic */
	{
		0x00003fff,
		0x00000000,
		0x00000000
	},
	/* Earth Magic */
	{
		0x00001fff,
		0x00000000,
		0x00000000
	},
	/* Wind Magic */
	{
		0x00000fff,
		0x00000000,
		0x00000000
	},
	/* Holy Magic */
	{
		0x00000fff,
		0x00fff000,
		0x00000000
	},
	/* Death Magic */
	{
		0x00001fff,
		0x01ffe000,
		0x00000000
	},
	/* Symbiotic Magic */
	{
		0x00007fff,
		0x00000000,
		0x00000000
	},
	/* Witch Magic */
	{
		0x000003ff,
		0x000ffc00,
		0x7ff00000
	},
	/* Drakonite Magic */
	{
		0x000000ff,
		0x00000000,
		0x00000000
	},
	/* Crusade Magic */
	{
		0x00000fff,
		0x01fff000,
		0x00000000
	},
};


#ifdef JP
cptr realm_names[] =
{
	"ˡʤ",
	"ƻ",
	"б",
	"",
	"",
	"",
	"",
	"Ź",
	"",
	"å",
	"ε",
	"˼",
	"",
};
#endif

#ifdef JP
cptr E_realm_names[]
#else
cptr realm_names[]
#endif
= {
	"none",
	"Magery",
	"Fire",
	"Aqua",
	"Earth",
	"Wind",
	"Holy",
	"Death",
	"Symbiotic",
	"Witch",
	"Drakonite",
	"Crusade",
	"unknown"
};

/*
 * Each chest has a certain set of traps, determined by pval
 * Each chest has a "pval" from 1 to the chest level (max 55)
 * If the "pval" is negative then the trap has been disarmed
 * The "pval" of a chest determines the quality of its treasure
 * Note that disarming a trap on a chest also removes the lock.
 */
int chest_traps[64] =
{
	0,                                      /* 0 == empty */
	(CHEST_POISON),
	(CHEST_LOSE_STR),
	(CHEST_LOSE_CON),
	(CHEST_LOSE_STR),
	(CHEST_LOSE_CON),                       /* 5 == best small wooden */
	0,
	(CHEST_ALARM),
	(CHEST_ALARM),
	(CHEST_LOSE_STR),
	(CHEST_LOSE_CON),
	(CHEST_POISON),
	(CHEST_SCATTER),
	(CHEST_LOSE_STR | CHEST_LOSE_CON),
	(CHEST_LOSE_STR | CHEST_LOSE_CON),
	(CHEST_SUMMON),                 /* 15 == best large wooden */
	0,
	(CHEST_ALARM),
	(CHEST_SCATTER),
	(CHEST_PARALYZE),
	(CHEST_LOSE_STR | CHEST_LOSE_CON),
	(CHEST_SUMMON),
	(CHEST_PARALYZE),
	(CHEST_LOSE_STR),
	(CHEST_LOSE_CON),
	(CHEST_EXPLODE),                        /* 25 == best small iron */
	0,
	(CHEST_E_SUMMON),
	(CHEST_POISON | CHEST_LOSE_CON),
	(CHEST_LOSE_STR | CHEST_LOSE_CON),
	(CHEST_EXPLODE | CHEST_SUMMON),
	(CHEST_BIRD_STORM),
	(CHEST_POISON | CHEST_SUMMON),
	(CHEST_E_SUMMON | CHEST_ALARM),
	(CHEST_EXPLODE),
	(CHEST_EXPLODE | CHEST_SUMMON), /* 35 == best large iron */
	0,
	(CHEST_SUMMON | CHEST_ALARM),
	(CHEST_EXPLODE),
	(CHEST_EXPLODE | CHEST_SUMMON),
	(CHEST_EXPLODE | CHEST_SUMMON),
	(CHEST_POISON | CHEST_PARALYZE),
	(CHEST_EXPLODE),
	(CHEST_BIRD_STORM),
	(CHEST_EXPLODE | CHEST_E_SUMMON | CHEST_ALARM),
	(CHEST_H_SUMMON),        /* 45 == best small steel */
	0,
	(CHEST_EXPLODE | CHEST_SUMMON | CHEST_ALARM),
	(CHEST_BIRD_STORM),
	(CHEST_RUNES_OF_EVIL),
	(CHEST_EXPLODE | CHEST_SUMMON | CHEST_ALARM),
	(CHEST_BIRD_STORM | CHEST_ALARM),
	(CHEST_H_SUMMON | CHEST_ALARM),
	(CHEST_RUNES_OF_EVIL),
	(CHEST_H_SUMMON | CHEST_SCATTER | CHEST_ALARM),
	(CHEST_RUNES_OF_EVIL | CHEST_EXPLODE),  /* 55 == best large steel */
	(CHEST_EXPLODE | CHEST_SUMMON),
	(CHEST_EXPLODE | CHEST_SUMMON),
	(CHEST_EXPLODE | CHEST_SUMMON),
	(CHEST_EXPLODE | CHEST_SUMMON),
	(CHEST_EXPLODE | CHEST_SUMMON),
	(CHEST_EXPLODE | CHEST_SUMMON),
	(CHEST_EXPLODE | CHEST_SUMMON),
	(CHEST_EXPLODE | CHEST_SUMMON),
};


/*
 * Hack -- the "basic" color names (see "TERM_xxx")
 */
cptr color_names[16] =
{
#ifdef JP
	"",
	"",
	"ĳ",
	"",
	"",
	"",
	"",
	"ῧ",
	"",
	"ĳ",
	"",
	"",
	"뤤",
	"뤤",
	"뤤",
	"ῧ",
#else
	"Dark",
	"White",
	"Slate",
	"Orange",
	"Red",
	"Green",
	"Blue",
	"Umber",
	"Light Dark",
	"Light Slate",
	"Violet",
	"Yellow",
	"Light Red",
	"Light Green",
	"Light Blue",
	"Light Umber",
#endif

};


/*
 * Abbreviations of healthy stats
 */
cptr stat_names[A_MAX] =
{
#ifdef JP
	" :", "ǽ :", " :", " :", "ѵ :", "̥ :"
#else
	"STR : ", "INT : ", "WIS : ", "DEX : ", "CON : ", "CHR : "
#endif

};

/*
 * Abbreviations of damaged stats
 */
cptr stat_names_reduced[A_MAX] =
{
#ifdef JP
	"x:", "ǽx:", "x:", "x:", "ѵx:", "̥x:"
#else
	"Str : ", "Int : ", "Wis : ", "Dex : ", "Con : ", "Chr : "
#endif

};


/*
 * Certain "screens" always use the main screen, including News, Birth,
 * Dungeon, Tomb-stone, High-scores, Macros, Colors, Visuals, Options.
 *
 * Later, special flags may allow sub-windows to "steal" stuff from the
 * main window, including File dump (help), File dump (artifacts, uniques),
 * Character screen, Small scale map, Previous Messages, Store screen, etc.
 *
 * The "ctrl-i" (tab) command flips the "Display inven/equip" and "Display
 * equip/inven" flags for all windows.
 *
 * The "ctrl-g" command (or pseudo-command) should perhaps grab a snapshot
 * of the main screen into any interested windows.
 */
cptr window_flag_desc[32] =
{
#ifdef JP
	"ʪ/",
#else
	"Display inven/equip",
#endif

#ifdef JP
	"/ʪ",
#else
	"Display equip/inven",
#endif

#ifdef JP
	"ʸ",
#else
	"Display spell list",
#endif

#ifdef JP
	"饯",
#else
	"Display character",
#endif

	NULL,
	NULL,
#ifdef JP
	"å",
#else
	"Display messages",
#endif

#ifdef JP
	"󥸥ο",
#else
	"Display overhead view",
#endif

#ifdef JP
	"󥹥λפ",
#else
	"Display monster recall",
#endif

#ifdef JP
	"ƥξܺ",
#else
	"Display object recall",
#endif

#ifdef JP
	"ʬμϤɽ", 
#else
	"Display dungeon view",
#endif

#ifdef JP
	"ǰ",
#else
	"Display snap-shot",
#endif

	NULL,
	NULL,
#ifdef JP
	"ܡå",
#else
	"Display borg messages",
#endif

#ifdef JP
	"ܡơ",
#else
	"Display borg status",
#endif

	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL
};


/*
 * Available Options
 *
 */
option_type option_info[] =
{
	/*** Input Options ***/

#ifdef JP
	{ &rogue_like_commands,         FALSE, OPT_PAGE_INPUT, 0, 0,
	"rogue_like_commands",          "֤Ѥ" },
#else
	{ &rogue_like_commands,         FALSE, OPT_PAGE_INPUT, 0, 0,
	"rogue_like_commands",          "Rogue-like commands" },
#endif


#ifdef JP
	{ &always_pickup,               FALSE, OPT_PAGE_INPUT, 0, 5,
	"always_pickup",                "˥ƥ򽦤" },
#else
	{ &always_pickup,               FALSE, OPT_PAGE_INPUT, 0, 5,
	"always_pickup",                "Pick things up by default" },
#endif


#ifdef JP
	{ &carry_query_flag,            FALSE, OPT_PAGE_INPUT, 0, 3,
	"carry_query_flag",             "ƥ򽦤˳ǧ" },
#else
	{ &carry_query_flag,            FALSE, OPT_PAGE_INPUT, 0, 3,
	"carry_query_flag",             "Prompt before picking things up" },
#endif


#ifdef JP
	{ &quick_messages,              TRUE,  OPT_PAGE_INPUT, 0, 1,
	"quick_messages",               "ååѤ" },
#else
	{ &quick_messages,              TRUE,  OPT_PAGE_INPUT, 0, 1,
	"quick_messages",               "Activate quick messages" },
#endif


#ifdef JP
	{ &auto_more,                   FALSE, OPT_PAGE_INPUT, 2, 6,
	"auto_more",                    "ԤʤϢ³ǥåɽ" },
#else
	{ &auto_more,                   FALSE, OPT_PAGE_INPUT, 2, 6,
	"auto_more",                    "Automatically clear '-more-' prompts" },
#endif


#ifdef JP
	{ &command_menu,                TRUE,  OPT_PAGE_INPUT, 2, 7,
	"command_menu",                 "˥塼ˤꥳޥͭˤ" },
#else
	{ &command_menu,                TRUE,  OPT_PAGE_INPUT, 2, 7,
	"command_menu",                 "Enable command selection menu" },
#endif


#ifdef JP
	{ &other_query_flag,            FALSE, OPT_PAGE_INPUT, 0, 2,
	"other_query_flag",             "ΥƥѤȤ˳ǧ" },
#else
	{ &other_query_flag,            FALSE, OPT_PAGE_INPUT, 0, 2,
	"other_query_flag",             "Prompt for floor item selection" },
#endif


#ifdef JP
	{ &use_old_target,              FALSE, OPT_PAGE_INPUT, 0, 4,
	"use_old_target",               "˰ΥåȤꤹ" },
#else
	{ &use_old_target,              FALSE, OPT_PAGE_INPUT, 0, 4,
	"use_old_target",               "Use old target by default" },
#endif


#ifdef JP
	{ &always_repeat,               TRUE,  OPT_PAGE_INPUT, 0, 6,
	"always_repeat",                "ޥɼư֤" },
#else
	{ &always_repeat,               TRUE,  OPT_PAGE_INPUT, 0, 6,
	"always_repeat",                "Repeat obvious commands" },
#endif


#ifdef JP
	{ &confirm_destroy,             FALSE, OPT_PAGE_INPUT, 5, 3,
	"confirm_destroy",              "̵͡פʥƥ˲ǧ" },
#else
	{ &confirm_destroy,             FALSE, OPT_PAGE_INPUT, 5, 3,
	"confirm_destroy",              "Prompt for destruction of known worthless items" },
#endif


#ifdef JP
	{ &confirm_wear,                TRUE,  OPT_PAGE_INPUT, 5, 4,
	"confirm_wear",                 "줿ʪǧ" },
#else
	{ &confirm_wear,                TRUE,  OPT_PAGE_INPUT, 5, 4,
	"confirm_wear",                 "Confirm to wear/wield known cursed items" },
#endif


#ifdef JP
	{ &confirm_quest,               TRUE,  OPT_PAGE_INPUT, 1, 9,
	"confirm_quest",                "ȤƳʤƨ˳ǧ" },
#else
	{ &confirm_quest,               TRUE,  OPT_PAGE_INPUT, 1, 9,
	"confirm_quest",                "Prompt before exiting a quest level" },
#endif


#ifdef JP
	{ &target_pet,                  FALSE, OPT_PAGE_INPUT, 2, 5,
	"target_pet",                   "ڥåȤ򥿡åȤˤ" },
#else
	{ &target_pet,                  FALSE, OPT_PAGE_INPUT, 2, 5,
	"target_pet",                   "Allow targetting pets" },
#endif


#ifdef JP
	{ &easy_open,                   TRUE,  OPT_PAGE_INPUT, 5, 7,
	"easy_open",                    "ưŪ˥ɥ򳫤" },
#else
	{ &easy_open,                   TRUE,  OPT_PAGE_INPUT, 5, 7,
	"easy_open",                    "Automatically open doors" },
#endif


#ifdef JP
	{ &easy_disarm,                 TRUE,  OPT_PAGE_INPUT, 5, 8,
	"easy_disarm",                  "ưŪ櫤" },
#else
	{ &easy_disarm,                 TRUE,  OPT_PAGE_INPUT, 5, 8,
	"easy_disarm",                  "Automatically disarm traps" },
#endif


#ifdef JP
	{ &easy_floor,                  FALSE, OPT_PAGE_INPUT, 5, 9,
	"easy_floor",                   "ǽŤʤäƥꥹȤ" },
#else
	{ &easy_floor,                  FALSE, OPT_PAGE_INPUT, 5, 9,
	"easy_floor",                   "Display floor stacks in a list" },
#endif


#ifdef JP
	{ &use_command,                 FALSE, OPT_PAGE_INPUT, 5, 10,
	"use_command",                  "ֻȤ(a)ץޥɤǥƥ򲿤ǤȤ" },
#else
	{ &use_command,                 FALSE, OPT_PAGE_INPUT, 5, 10,
	"use_command",                  "Allow unified use command" },
#endif


#ifdef JP
	{ &over_exert,                  FALSE, OPT_PAGE_INPUT, 0, 29,
	"over_exert",                   "MP­ʤƤˡĩ魯" },
#else
	{ &over_exert,                  FALSE, OPT_PAGE_INPUT, 0, 29,
	"over_exert",                   "Allow casting spells when short of mana" },
#endif


	/*** Map Screen Options ***/

#ifdef JP
	{ &center_player,               FALSE, OPT_PAGE_MAPSCREEN, 5, 11,
	"center_player",                "˥ץ쥤䡼濴֤(*٤*)" },
#else
	{ &center_player,               FALSE, OPT_PAGE_MAPSCREEN, 5, 11,
	"center_player",                "Center map while walking (*slow*)" },
#endif


#ifdef JP
	{ &center_running,              TRUE,  OPT_PAGE_MAPSCREEN, 5, 12,
	"center_running",               "äƤǤ濴֤" },
#else
	{ &center_running,              TRUE,  OPT_PAGE_MAPSCREEN, 5, 12,
	"center_running",               "Centering even while running" },
#endif


#ifdef JP
	{ &view_yellow_lite,            TRUE,  OPT_PAGE_MAPSCREEN, 1, 28,
	"view_yellow_lite",              "ϰϤ̤ʿɽ" },
#else
	{ &view_yellow_lite,            TRUE,  OPT_PAGE_MAPSCREEN, 1, 28,
	"view_yellow_lite",             "Use special colors for torch-lit grids" },
#endif


#ifdef JP
	{ &view_bright_lite,            TRUE,  OPT_PAGE_MAPSCREEN, 1, 29,
	"view_bright_lite",             "볦ϰϤ̤ʿɽ" },
#else
	{ &view_bright_lite,            TRUE,  OPT_PAGE_MAPSCREEN, 1, 29,
	"view_bright_lite",             "Use special colors for 'viewable' grids" },
#endif


#ifdef JP
	{ &view_granite_lite,           TRUE,  OPT_PAGE_MAPSCREEN, 1, 30,
	"view_granite_lite",            "ɤ̤ʿɽ(Ť)" },
#else
	{ &view_granite_lite,           TRUE,  OPT_PAGE_MAPSCREEN, 1, 30,
	"view_granite_lite",            "Use special colors for wall grids (slow)" },
#endif


#ifdef JP
	{ &view_special_lite,           TRUE,  OPT_PAGE_MAPSCREEN, 1, 31,
	"view_special_lite",            "̤ʿɽ(Ť)" },
#else
	{ &view_special_lite,           TRUE,  OPT_PAGE_MAPSCREEN, 1, 31,
	"view_special_lite",            "Use special colors for floor grids (slow)" },
#endif

#ifdef JP
	{ &new_ascii_graphics,          FALSE, OPT_PAGE_MAPSCREEN, 2, 4,
	"new_ascii_graphics",           "ϷŤϤäɽ" },
#else
	{ &new_ascii_graphics,          FALSE, OPT_PAGE_MAPSCREEN, 2, 4,
	"new_ascii_graphics",           "Show a clear contrast between light and dark" },
#endif

#ifdef JP
	{ &use_fake_monochrome,         FALSE, OPT_PAGE_MAPSCREEN, 4, 2,
	"use_fake_monochrome",          "̤ʾǥΥɽˤ" },
#else
	{ &use_fake_monochrome,         FALSE, OPT_PAGE_MAPSCREEN, 4, 2,
	"use_fake_monochrome",          "Use monochrome display on special occasion" },
#endif


#ifdef JP
	{ &view_perma_grids,            TRUE,  OPT_PAGE_MAPSCREEN, 1, 6,
	"view_perma_grids",             "뤤ϤΤޤޤˤ" },
#else
	{ &view_perma_grids,            TRUE,  OPT_PAGE_MAPSCREEN, 1, 6,
	"view_perma_grids",             "Map remembers all perma-lit grids" },
#endif


#ifdef JP
	{ &view_torch_grids,            FALSE, OPT_PAGE_MAPSCREEN, 1, 7,
	"view_torch_grids",             "ǾȤ餷ϤΤޤޤˤ" },
#else
	{ &view_torch_grids,            FALSE, OPT_PAGE_MAPSCREEN, 1, 7,
	"view_torch_grids",             "Map remembers all torch-lit grids" },
#endif


#ifdef JP
	{ &view_unsafe_grids,            FALSE, OPT_PAGE_MAPSCREEN, 1, 8,
	"view_unsafe_grids",             "ȥå״κѤߤǤʤɽ" },
#else
	{ &view_unsafe_grids,            FALSE, OPT_PAGE_MAPSCREEN, 1, 8,
	"view_unsafe_grids",             "Map marked by detect traps" },
#endif


#ifdef JP
	{ &view_reduce_view,            FALSE, OPT_PAGE_MAPSCREEN, 1, 17,
	"view_reduce_view",             "Ǥϻ򶹤" },
#else
	{ &view_reduce_view,            FALSE, OPT_PAGE_MAPSCREEN, 1, 17,
	"view_reduce_view",             "Reduce view-radius in town" },
#endif


#ifdef JP
	{ &fresh_before,                TRUE,  OPT_PAGE_MAPSCREEN, 1, 23,
	"fresh_before",                 "Ϣ³ޥ˲̤褷³" },
#else
	{ &fresh_before,                TRUE,  OPT_PAGE_MAPSCREEN, 1, 23,
	"fresh_before",                 "Flush output while continuous command" },
#endif


#ifdef JP
	{ &fresh_after,                 FALSE, OPT_PAGE_MAPSCREEN, 1, 24,
	"fresh_after",                  "ޥɸ˲̤˺褷³" },
#else
	{ &fresh_after,                 FALSE, OPT_PAGE_MAPSCREEN, 1, 24,
	"fresh_after",                  "Flush output after monster's move" },
#endif


#ifdef JP
	{ &fresh_message,               FALSE, OPT_PAGE_MAPSCREEN, 1, 25,
	"fresh_message",                "åθ˲̤褹" },
#else
	{ &fresh_message,               FALSE, OPT_PAGE_MAPSCREEN, 1, 25,
	"fresh_message",                "Flush output after every message" },
#endif


#ifdef JP
	{ &hilite_player,               FALSE,  OPT_PAGE_MAPSCREEN, 1, 27,
	"hilite_player",                "ץ쥤䡼˥碌" },
#else
	{ &hilite_player,               FALSE,  OPT_PAGE_MAPSCREEN, 1, 27,
	"hilite_player",                "Hilite the player with the cursor" },
#endif


#ifdef JP
	{ &display_path,                FALSE, OPT_PAGE_MAPSCREEN, 2, 8,
	"display_path",                 "ˡεפɽ" },
#else
	{ &display_path,                FALSE, OPT_PAGE_MAPSCREEN, 2, 8,
	"display_path",                 "Display actual path before shooting" },
#endif


	/*** Text Display Options ***/

#ifdef JP
	{ &plain_pickup,                FALSE, OPT_PAGE_TEXT, 6, 6,
	"plain_pickup",                 "ֽäץåά" },
#else
	{ &plain_pickup,                FALSE, OPT_PAGE_JAPANESE_ONLY, 6, 6,
	"plain_pickup",                 "Plain pickup messages(japanese only)" },
#endif


#ifdef JP
	{ &always_show_list,            TRUE,  OPT_PAGE_TEXT, 4, 0,
	"always_show_list",             "ˤϾ˰ɽ" },
#else
	{ &always_show_list,            TRUE,  OPT_PAGE_TEXT, 4, 0,
	"always_show_list",             "Always show list at first when select items" },
#endif


#ifdef JP
	{ &depth_in_feet,               FALSE, OPT_PAGE_TEXT, 0, 7,
	"depth_in_feet",                "󥸥οեȤɽ" },
#else
	{ &depth_in_feet,               FALSE, OPT_PAGE_TEXT, 0, 7,
	"depth_in_feet",                "Show dungeon level in feet" },
#endif


#ifdef JP
	{ &show_damage,                  TRUE, OPT_PAGE_TEXT, 4, 1,
	"show_damage",                   "/Ϳ᡼ɽ" },
#else
	{ &show_damage,                  TRUE, OPT_PAGE_TEXT, 4, 1,
	"show_damage",                   "Show done/taken damage" },
#endif


#ifdef JP
	{ &show_labels,                 TRUE,  OPT_PAGE_TEXT, 0, 10,
	"show_labels",                  "ɽ" },
#else
	{ &show_labels,                 TRUE,  OPT_PAGE_TEXT, 0, 10,
	"show_labels",                  "Show labels in object listings" },
#endif


#ifdef JP
	{ &show_weights,                TRUE,  OPT_PAGE_TEXT, 0, 11,
	"show_weights",                 "ƥǽ̤ɽ" },
#else
	{ &show_weights,                TRUE,  OPT_PAGE_TEXT, 0, 11,
	"show_weights",                 "Show weights in object listings" },
#endif


#ifdef JP
	{ &show_item_graph,             TRUE,  OPT_PAGE_TEXT, 2, 0,
	"show_item_graph",              "ƥΥܥɽ" },
#else
	{ &show_item_graph,             TRUE,  OPT_PAGE_TEXT, 2, 0,
	"show_item_graph",              "Show items graphics" },
#endif

#ifdef JP
	{ &equippy_chars,               TRUE,  OPT_PAGE_TEXT, 1, 12,
	"equippy_chars",                "ơʸɽ" },
#else
	{ &equippy_chars,               TRUE,  OPT_PAGE_TEXT, 1, 12,
	"equippy_chars",                "Display 'equippy' chars" },
#endif


#ifdef JP
	{ &display_mutations,           FALSE, OPT_PAGE_TEXT, 5, 0,
	"display_mutations",            "'C'ޥɤѰۤɽ" },
#else
	{ &display_mutations,           FALSE, OPT_PAGE_TEXT, 5, 0,
	"display_mutations",            "Display mutations in 'C'haracter Display" },
#endif


#ifdef JP
	{ &compress_savefile,           FALSE, OPT_PAGE_TEXT, 1, 26,
	"compress_savefile",            "֡եΥå򰵽̤" },
#else
	{ &compress_savefile,           FALSE, OPT_PAGE_TEXT, 1, 26,
	"compress_savefile",            "Compress messages in savefiles" },
#endif


#ifdef JP
	{ &abbrev_extra,                FALSE, OPT_PAGE_TEXT, 2, 10,
	"abbrev_extra",                 "ƥɲ/ǽϤάΤ" },
#else
	{ &abbrev_extra,                FALSE, OPT_PAGE_TEXT, 2, 10,
	"abbrev_extra",                 "Describe obj's extra resistances by abbreviation" },
#endif

#ifdef JP
	{ &abbrev_all,                  FALSE, OPT_PAGE_TEXT, 2, 11,
	"abbrev_all",                   "ƥƤ/ǽϤάΤ" },
#else
	{ &abbrev_all,                  FALSE, OPT_PAGE_TEXT, 2, 11,
	"abbrev_all",                   "Describe obj's all resistances by abbreviation" },
#endif


	/*** Game-Play Options ***/

#ifdef JP
	{ &stack_force_notes,           TRUE,  OPT_PAGE_GAMEPLAY, 0, 8,
	"stack_force_notes",            "ۤʤäΥƥޤȤ" },
#else
	{ &stack_force_notes,           TRUE,  OPT_PAGE_GAMEPLAY, 0, 8,
	"stack_force_notes",            "Merge inscriptions when stacking" },
#endif


#ifdef JP
	{ &stack_force_costs,           FALSE, OPT_PAGE_GAMEPLAY, 0, 9,
	"stack_force_costs",            "ۤʤɽΥƥޤȤ" },
#else
	{ &stack_force_costs,           FALSE, OPT_PAGE_GAMEPLAY, 0, 9,
	"stack_force_costs",            "Merge discounts when stacking" },
#endif


#ifdef JP
	{ &expand_list,                 TRUE,  OPT_PAGE_GAMEPLAY, 1, 5,
	"expand_list",                  "ְץޥɤĥ" },
#else
	{ &expand_list,                 TRUE,  OPT_PAGE_GAMEPLAY, 1, 5,
	"expand_list",                  "Expand the power of the list commands" },
#endif


#ifdef JP
	{ &small_levels,                TRUE,  OPT_PAGE_GAMEPLAY, 0, 30,
	"small_levels",                 "˾եǽˤ" },
#else
	{ &small_levels,                TRUE,  OPT_PAGE_GAMEPLAY, 0, 30,
	"small_levels",                 "Allow unusually small dungeon levels" },
#endif


#ifdef JP
	{ &always_small_levels,         FALSE, OPT_PAGE_GAMEPLAY, 2, 3,
	"always_small_levels",          "˾ե" },
#else
	{ &always_small_levels,         FALSE, OPT_PAGE_GAMEPLAY, 2, 3,
	"always_small_levels",          "Always create unusually small dungeon levels" },
#endif


#ifdef JP
	{ &empty_levels,                TRUE,  OPT_PAGE_GAMEPLAY, 0, 31,
	"empty_levels",                 "äݤΡ֥꡼ʡץ٥ǽˤ" },
#else
	{ &empty_levels,                TRUE,  OPT_PAGE_GAMEPLAY, 0, 31,
	"empty_levels",                 "Allow empty 'arena' levels" },
#endif


#ifdef JP
	{ &last_words,                  TRUE, OPT_PAGE_GAMEPLAY, 0, 28,
	"last_words",                   "饯Τ" },
#else
	{ &last_words,                  TRUE, OPT_PAGE_GAMEPLAY, 0, 28,
	"last_words",                   "Leave last words when your character dies" },
#endif


#ifdef JP
	{ &allow_debug_opts,            FALSE, OPT_PAGE_GAMEPLAY, 6, 11,
	"allow_debug_opts",             "ǥХå/ץĤ" },
#else
	{ &allow_debug_opts,            FALSE, OPT_PAGE_GAMEPLAY, 6, 11,
	"allow_debug_opts",             "Allow use of debug/cheat options" },
#endif


	/*** Disturbance Options ***/

#ifdef JP
	{ &find_ignore_stairs,          FALSE, OPT_PAGE_DISTURBANCE, 0, 16,
	"find_ignore_stairs",           "ʤ̲᤹" },
#else
	{ &find_ignore_stairs,          FALSE, OPT_PAGE_DISTURBANCE, 0, 16,
	"find_ignore_stairs",           "Run past stairs" },
#endif


#ifdef JP
	{ &find_ignore_doors,           TRUE,  OPT_PAGE_DISTURBANCE, 0, 17,
	"find_ignore_doors",            "ɥ̲᤹" },
#else
	{ &find_ignore_doors,           TRUE,  OPT_PAGE_DISTURBANCE, 0, 17,
	"find_ignore_doors",            "Run through open doors" },
#endif


#ifdef JP
	{ &find_cut,                    FALSE, OPT_PAGE_DISTURBANCE, 0, 18,
	"find_cut",                     "ʤѤФ˺ûΥ̲᤹" },
#else
	{ &find_cut,                    FALSE, OPT_PAGE_DISTURBANCE, 0, 18,
	"find_cut",                     "Run past known corners" },
#endif


#ifdef JP
	{ &check_abort,                 TRUE,  OPT_PAGE_DISTURBANCE, 1, 18,
	"check_abort",                  "Ϣ³ޥɤϥϤǤ" },
#else
	{ &check_abort,                 TRUE,  OPT_PAGE_DISTURBANCE, 1, 18,
	"check_abort",                  "Check for user abort while continuous command" },
#endif


#ifdef JP
	{ &flush_failure,               TRUE,  OPT_PAGE_DISTURBANCE, 1, 20,
	"flush_failure",                "͡ʥߥȯϤ򥯥ꥢ" },
#else
	{ &flush_failure,               TRUE,  OPT_PAGE_DISTURBANCE, 1, 20,
	"flush_failure",                "Flush input on various failures" },
#endif


#ifdef JP
	{ &flush_disturb,               FALSE, OPT_PAGE_DISTURBANCE, 1, 21,
	"flush_disturb",                "㳲ȯϤ򥯥ꥢ" },
#else
	{ &flush_disturb,               FALSE, OPT_PAGE_DISTURBANCE, 1, 21,
	"flush_disturb",                "Flush input whenever disturbed" },
#endif


#ifdef JP
	{ &disturb_move,                FALSE,  OPT_PAGE_DISTURBANCE, 0, 20,
	"disturb_move",                 "ɤΥ󥹥ưƤưߤ" },
#else
	{ &disturb_move,                FALSE,  OPT_PAGE_DISTURBANCE, 0, 20,
	"disturb_move",                 "Disturb whenever any monster moves" },
#endif


#ifdef JP
	{ &disturb_high,                FALSE,  OPT_PAGE_DISTURBANCE, 1, 3,
	"disturb_high",                 "٥ι⤤󥹥ưưߤ"},
#else
	{ &disturb_high,                FALSE,  OPT_PAGE_DISTURBANCE, 1, 3,
	"disturb_high",                 "Disturb whenever high-level monster moves" },
#endif


#ifdef JP
	{ &disturb_near,                TRUE,  OPT_PAGE_DISTURBANCE, 0, 21,
	"disturb_near",                 "볦Υ󥹥ưưߤ" },
#else
	{ &disturb_near,                TRUE,  OPT_PAGE_DISTURBANCE, 0, 21,
	"disturb_near",                 "Disturb whenever viewable monster moves" },
#endif


#ifdef JP
	{ &disturb_pets,                FALSE, OPT_PAGE_DISTURBANCE, 5, 6,
	"disturb_pets",                 "볦ΥڥåȤưưߤ" },
#else
	{ &disturb_pets,                FALSE, OPT_PAGE_DISTURBANCE, 5, 6,
	"disturb_pets",                 "Disturb when visible pets move" },
#endif


#ifdef JP
	{ &disturb_panel,               TRUE,  OPT_PAGE_DISTURBANCE, 0, 22,
	"disturb_panel",                "̥˹ưߤ" },
#else
	{ &disturb_panel,               TRUE,  OPT_PAGE_DISTURBANCE, 0, 22,
	"disturb_panel",                "Disturb whenever map panel changes" },
#endif


#ifdef JP
	{ &disturb_state,               TRUE,  OPT_PAGE_DISTURBANCE, 0, 23,
	"disturb_state",                "ʬΥơѲ˹ưߤ" },
#else
	{ &disturb_state,               TRUE,  OPT_PAGE_DISTURBANCE, 0, 23,
	"disturb_state",                "Disturb whenever player state changes" },
#endif


#ifdef JP
	{ &disturb_minor,               TRUE,  OPT_PAGE_DISTURBANCE, 0, 24,
	"disturb_minor",                "٤ʤȤƤưߤ" },
#else
	{ &disturb_minor,               TRUE,  OPT_PAGE_DISTURBANCE, 0, 24,
	"disturb_minor",                "Disturb whenever boring things happen" },
#endif


#ifdef JP
	{ &ring_bell,                   FALSE, OPT_PAGE_DISTURBANCE, 0, 14,
	"ring_bell",                    "顼˥ӡײĤ餹" },
#else
	{ &ring_bell,                   FALSE, OPT_PAGE_DISTURBANCE, 0, 14,
	"ring_bell",                    "Audible bell (on errors, etc)" },
#endif


#ifdef JP
	{ &disturb_trap_detect,         TRUE, OPT_PAGE_DISTURBANCE, 0, 27,
	"disturb_trap_detect",          "ȥå״ϰϳ˽Фľ˹ưߤ" },
#else
	{ &disturb_trap_detect,         TRUE, OPT_PAGE_DISTURBANCE, 0, 27,
	"disturb_trap_detect",          "Disturb when leaving trap detected area"},
#endif

#ifdef JP
	{ &alert_trap_detect,           FALSE, OPT_PAGE_DISTURBANCE, 0, 25,
	"alert_trap_detect",            "ȥå״ϰϳ˽Фľ˷ٹ𤹤" },
#else
	{ &alert_trap_detect,           FALSE, OPT_PAGE_DISTURBANCE, 0, 25,
	"alert_trap_detect",            "Alert when leaving trap detected area"},
#endif


	/*** Birth Options ***/

#ifdef JP
	{ &manual_haggle,               FALSE, OPT_PAGE_BIRTH, 1, 0,
	"manual_haggle",                "ŹڤĤ򤹤" },
#else
	{ &manual_haggle,               FALSE, OPT_PAGE_BIRTH, 1, 0,
	"manual_haggle",                "Manually haggle in stores" },
#endif


#ifdef JP
	{ &easy_band,                   FALSE, OPT_PAGE_BIRTH, 6, 31,
	"easy_band",                    "鿴Ѵñ⡼(*)" },
#else
	{ &easy_band,                   FALSE, OPT_PAGE_BIRTH, 6, 31,
	"easy_band",                    "Easy Mode (*)" },
#endif


#ifdef JP
	{ &smart_learn,                 TRUE,  OPT_PAGE_BIRTH, 1, 14,
	"smart_learn",                  "󥹥ϼԤؽ(*)" },
#else
	{ &smart_learn,                 TRUE,  OPT_PAGE_BIRTH, 1, 14,
	"smart_learn",                  "Monsters learn from their mistakes (*)" },
#endif


#ifdef JP
	{ &smart_cheat,                 FALSE, OPT_PAGE_BIRTH, 1, 15,
	"smart_cheat",                  "󥹥ϥץ쥤䡼μߤͤ(*)" },
#else
	{ &smart_cheat,                 FALSE, OPT_PAGE_BIRTH, 1, 15,
	"smart_cheat",                  "Monsters exploit players weaknesses (*)" },
#endif


#ifdef JP
	{ &ironman_rooms,                FALSE, OPT_PAGE_BIRTH, 6, 12,
	"ironman_rooms",                 "Ŵѡ˾̤Ǥʤ(*)" },
#else
	{ &ironman_rooms,               FALSE, OPT_PAGE_BIRTH, 6, 12,
	"ironman_rooms",                "Always generate very unusual rooms (*)" },
#endif


#ifdef JP
	{ &left_hander,                 FALSE, OPT_PAGE_BIRTH, 6, 13,
	"left_hander",                  "Ǥ" },
#else
	{ &left_hander,                 FALSE, OPT_PAGE_BIRTH, 6, 13,
	"left_hander",                  "Left-Hander" },
#endif


#ifdef JP
	{ &preserve_mode,               TRUE,  OPT_PAGE_BIRTH, 6, 14,
	"preserve_mode",                "ΥƥƨƤ(*)" },
#else
	{ &preserve_mode,               TRUE,  OPT_PAGE_BIRTH, 6, 14,
	"preserve_mode",                "Preserve artifacts (*)" },
#endif


#ifdef JP
	{ &powerup_home,                TRUE,  OPT_PAGE_BIRTH, 4, 3,
	"powerup_home",                 "椬Ȥĥ(*)" },

#else
	{ &powerup_home,                TRUE,  OPT_PAGE_BIRTH, 4, 3,
	"powerup_home",                 "Increase capacity of your home (*)" },
#endif


	/*** Easy Object Auto-Destroyer ***/

#ifdef JP
	{ &destroy_items,               FALSE, OPT_PAGE_AUTODESTROY, 7, 0,
	"destroy_items",                "ƥδʰ׼ư˲Ѥ" },
#else
	{ &destroy_items,               FALSE, OPT_PAGE_AUTODESTROY, 7, 0,
	"destroy_items",                "Use easy auto-destroyer" },
#endif

#ifdef JP
	{ &destroy_feeling,               FALSE, OPT_PAGE_AUTODESTROY, 7, 8,
	"destroy_feeling",                "ʰ״ꤷȤư˲ŬѤ" },
#else
	{ &destroy_feeling,               FALSE, OPT_PAGE_AUTODESTROY, 7, 8,
	"destroy_feeling",                "Apply auto-destroy as sense feeling" },
#endif

#ifdef JP
	{ &destroy_identify,               FALSE, OPT_PAGE_AUTODESTROY, 7, 9,
	"destroy_identify",                "ꤷȤư˲ŬѤ" },
#else
	{ &destroy_identify,               FALSE, OPT_PAGE_AUTODESTROY, 7, 9,
	"destroy_identify",                "Apply auto-destroy as identify an item" },
#endif

#ifdef JP
	{ &leave_worth,                 TRUE, OPT_PAGE_AUTODESTROY, 7, 2,
	"leave_worth",                  "ͤ륢ƥϲʤ" },
#else
	{ &leave_worth,                 TRUE, OPT_PAGE_AUTODESTROY, 7, 2,
	"leave_worth",                  "Auto-destroyer leaves known worthy items" },
#endif

#ifdef JP
	{ &leave_equip,                 FALSE, OPT_PAGE_AUTODESTROY, 7, 3,
	"leave_equip",                  "/ɶϲʤ" },
#else
	{ &leave_equip,                 FALSE, OPT_PAGE_AUTODESTROY, 7, 3,
	"leave_equip",                  "Auto-destroyer leaves weapons and armour" },
#endif

#ifdef JP
	{ &leave_chest,                 TRUE, OPT_PAGE_AUTODESTROY, 7, 7,
	"leave_chest",                  "ƤʤȢϲʤ" },
#else
	{ &leave_chest,                 TRUE, OPT_PAGE_AUTODESTROY, 7, 7,
	"leave_chest",                  "Auto-destroyer leaves closed chests" },
#endif

#ifdef JP
	{ &leave_wanted,                TRUE, OPT_PAGE_AUTODESTROY, 7, 4,
	"leave_wanted",                 "޶λ/ϲʤ" },
#else
	{ &leave_wanted,                TRUE, OPT_PAGE_AUTODESTROY, 7, 4,
	"leave_wanted",                 "Auto-destroyer leaves wanted corpses" },
#endif

#ifdef JP
	{ &leave_corpse,                FALSE, OPT_PAGE_AUTODESTROY, 7, 5,
	"leave_corpse",                 "/ϲʤ" },
#else
	{ &leave_corpse,                FALSE, OPT_PAGE_AUTODESTROY, 7, 5,
	"leave_corpse",                 "Auto-destroyer leaves corpses and skeletons" },
#endif

#ifdef JP
	{ &leave_junk,                  FALSE, OPT_PAGE_AUTODESTROY, 7, 6,
	"leave_junk",                   "餯ϲʤ" },
#else
	{ &leave_junk,                  FALSE, OPT_PAGE_AUTODESTROY, 7, 6,
	"leave_junk",                   "Auto-destroyer leaves junk" },
#endif

#ifdef JP
	{ &leave_special,               TRUE, OPT_PAGE_AUTODESTROY, 7, 1,
	"leave_special",                "²/Ȥ̤ɬפʥƥϲʤ" },
#else
	{ &leave_special,               TRUE, OPT_PAGE_AUTODESTROY, 7, 1,
	"leave_special",                "Auto-destroyer leaves items your race/class needs" },
#endif


	/*** Play-record Options ***/

#ifdef JP
	{ &record_fix_art,              TRUE, OPT_PAGE_PLAYRECORD, 4, 11,
	"record_fix_art",               "ꥢƥեȤϿ" },
#else
	{ &record_fix_art,              TRUE, OPT_PAGE_PLAYRECORD, 4, 11,
	"record_fix_art",               "Record fixed artifacts" },
#endif

#ifdef JP
	{ &record_rand_art,             TRUE, OPT_PAGE_PLAYRECORD, 4, 12,
	"record_rand_art",              "ॢƥեȤϿ" },
#else
	{ &record_rand_art,             TRUE, OPT_PAGE_PLAYRECORD, 4, 12,
	"record_rand_art",              "Record random artifacts" },
#endif

#ifdef JP
	{ &record_destroy_uniq,         TRUE, OPT_PAGE_PLAYRECORD, 4, 13,
	"record_destroy_uniq",          "ˡ󥹥ݤȤϿ" },
#else
	{ &record_destroy_uniq,         TRUE, OPT_PAGE_PLAYRECORD, 4, 13,
	"record_destroy_uniq",          "Record when destroy unique monster" },
#endif

#ifdef JP
	{ &record_fix_quest,            TRUE, OPT_PAGE_PLAYRECORD, 4, 14,
	"record_fix_quest",             "ꥯȤãϿ" },
#else
	{ &record_fix_quest,            TRUE, OPT_PAGE_PLAYRECORD, 4, 14,
	"record_fix_quest",             "Record fixed quests" },
#endif

#ifdef JP
	{ &record_rand_quest,           TRUE, OPT_PAGE_PLAYRECORD, 4, 15,
	"record_rand_quest",            "९ȤãϿ" },
#else
	{ &record_rand_quest,           TRUE, OPT_PAGE_PLAYRECORD, 4, 15,
	"record_rand_quest",            "Record random quests" },
#endif

#ifdef JP
	{ &record_maxdeapth,            TRUE, OPT_PAGE_PLAYRECORD, 4, 16,
	"record_maxdeapth",             "ǿ򹹿Ȥ˵Ͽ" },
#else
	{ &record_maxdeapth,            TRUE, OPT_PAGE_PLAYRECORD, 4, 16,
	"record_maxdeapth",             "Record movements to deepest level" },
#endif

#ifdef JP
	{ &record_stair,                TRUE, OPT_PAGE_PLAYRECORD, 4, 17,
	"record_stair",                 "ΰưϿ" },
#else
	{ &record_stair,                TRUE, OPT_PAGE_PLAYRECORD, 4, 17,
	"record_stair",                 "Record recall and stair movements" },
#endif

#ifdef JP
	{ &record_buy,                  TRUE, OPT_PAGE_PLAYRECORD, 4, 18,
	"record_buy",                   "ƥιϿ" },
#else
	{ &record_buy,                  TRUE, OPT_PAGE_PLAYRECORD, 4, 18,
	"record_buy",                   "Record purchased items" },
#endif

#ifdef JP
	{ &record_sell,                 FALSE, OPT_PAGE_PLAYRECORD, 4, 19,
	"record_sell",                  "ƥѤϿ" },
#else
	{ &record_sell,                 FALSE, OPT_PAGE_PLAYRECORD, 4, 19,
	"record_sell",                  "Record sold items" },
#endif

#ifdef JP
	{ &record_danger,               TRUE, OPT_PAGE_PLAYRECORD, 4, 20,
	"record_danger",                "ԥˤʤäȤϿ" },
#else
	{ &record_danger,               TRUE, OPT_PAGE_PLAYRECORD, 4, 20,
	"record_danger",                "Record hitpoint warning" },
#endif

#ifdef JP
	{ &record_arena,                TRUE, OPT_PAGE_PLAYRECORD, 4, 21,
	"record_arena",                 "꡼ʤǤξϿ" },
#else
	{ &record_arena,                TRUE, OPT_PAGE_PLAYRECORD, 4, 21,
	"record_arena",                 "Record arena victories" },
#endif

#ifdef JP
	{ &record_ident,                TRUE, OPT_PAGE_PLAYRECORD, 4, 22,
	"record_ident",                 "̤ȽΥƥμ̤Ͽ" },
#else
	{ &record_ident,                TRUE, OPT_PAGE_PLAYRECORD, 4, 22,
	"record_ident",                 "Record first identified items" },
#endif

#ifdef JP
	{ &record_named_pet,            FALSE, OPT_PAGE_PLAYRECORD, 4, 23,
	"record_named_pet",             "̾ĤڥåȤξϿ" },
#else
	{ &record_named_pet,            FALSE, OPT_PAGE_PLAYRECORD, 4, 23,
	"record_named_pet",             "Record informations of named pets" },
#endif


	/*** End of Table ***/

	{ NULL,                         0, 0, 0, 0,
	NULL,                           NULL }
};




martial_arts ma_blows[MAX_MA] =
{
#ifdef JP
	{ "%s򲥤ä",                 SKILL_LEVEL_BEGINNER,  0,  1,  5,  0 },
	{ "%s򽳤ä",                 SKILL_LEVEL_BEGINNER,  0,  1,  7,  0 },
	{ "%sͤ򤯤路",   SKILL_LEVEL_NOVICE,    0,  1,  9,  0 },
	{ "%sɨ򤯤路",     SKILL_LEVEL_NOVICE,    5,  2,  4, MA_KNEE },
	{ "%sɪǤ򤯤路",     SKILL_LEVEL_NOVICE,    5,  1, 12,  0 },
	{ "%sꤷ",             SKILL_LEVEL_AVERAGE,  10,  2,  6,  0 },
	{ "%s򽳤ä",                 SKILL_LEVEL_AVERAGE,  10,  3,  6, MA_SLOW },
	{ "%s˥åѡ򤯤路",   SKILL_LEVEL_AVERAGE,  12,  5,  5,  6 },
	{ "%sʽ򤯤路",   SKILL_LEVEL_SKILLED,  15,  5,  6,  8 },
	{ "%sǭ޷򤯤路",     SKILL_LEVEL_SKILLED,  20,  5,  8,  0 },
	{ "%sķӤ򤯤路",     SKILL_LEVEL_SKILLED,  25,  6,  8, 10 },
	{ "%s޽򤯤路",     SKILL_LEVEL_EXPERT,   25,  7,  9,  0 },
	{ "%s˲󤷽򤯤路",   SKILL_LEVEL_EXPERT,   30,  8, 10, 10 },
	{ "%sŴ򤯤路",     SKILL_LEVEL_EXPERT,   35,  8, 11, 10 },
	{ "%sӤ򤯤路",     SKILL_LEVEL_MASTER,   35,  8, 12, 12 },
	{ "%sȾ򤯤路",     SKILL_LEVEL_MASTER,   35,  9, 12, 16 },
	{ "%sƲήӤ򤯤路", SKILL_LEVEL_MASTER,   40, 10, 13, 18 },
#else
	{ "You punch %s.",                               SKILL_LEVEL_BEGINNER,  0,  1,  5,  0 },
	{ "You kick %s.",                                SKILL_LEVEL_BEGINNER,  0,  1,  7,  0 },
	{ "You strike %s.",                              SKILL_LEVEL_NOVICE,    0,  1,  9,  0 },
	{ "You hit %s with your knee.",                  SKILL_LEVEL_NOVICE,    5,  2,  4, MA_KNEE },
	{ "You hit %s with your elbow.",                 SKILL_LEVEL_NOVICE,    5,  1, 12,  0 },
	{ "You butt %s.",                                SKILL_LEVEL_AVERAGE,  10,  2,  6,  0 },
	{ "You kick %s.",                                SKILL_LEVEL_AVERAGE,  10,  3,  6, MA_SLOW },
	{ "You uppercut %s.",                            SKILL_LEVEL_AVERAGE,  12,  5,  5,  6 },
	{ "You double-kick %s.",                         SKILL_LEVEL_SKILLED,  15,  5,  6,  8 },
	{ "You hit %s with a Cat's Claw.",               SKILL_LEVEL_SKILLED,  20,  5,  8,  0 },
	{ "You hit %s with a jump kick.",                SKILL_LEVEL_SKILLED,  25,  6,  8, 10 },
	{ "You hit %s with an Eagle's Claw.",            SKILL_LEVEL_EXPERT,   25,  7,  9,  0 },
	{ "You hit %s with a circle kick.",              SKILL_LEVEL_EXPERT,   30,  8, 10, 10 },
	{ "You hit %s with an Iron Fist.",               SKILL_LEVEL_EXPERT,   35,  8, 11, 10 },
	{ "You hit %s with a flying kick.",              SKILL_LEVEL_MASTER,   35,  8, 12, 12 },
	{ "You hit %s with a Half-moon Kick.",           SKILL_LEVEL_MASTER,   35,  9, 12, 16 },
	{ "You hit %s with a Kingly Shooting Star Kick.",SKILL_LEVEL_MASTER,   40, 10, 13, 18 },
#endif

};

/*
 * Table of game-generated inscriptions (indexed by the defines in
 * defines.h). -- RG
 */
cptr game_inscriptions[] =
{
	NULL,            /* FEEL_NONE */
#ifdef JP
	"Ƥ",    /* FEEL_BROKEN */
	"",      /* FEEL_TERRIBLE */
	"̵",        /* FEEL_WORTHLESS */
	"Ƥ",  /* FEEL_CURSED */
	"ʾ",      /* FEEL_UNCURSED */
	"",            /* FEEL_AVERAGE */
	"",          /* FEEL_GOOD */
	"",        /* FEEL_EXCELLENT */
	"",        /* FEEL_SPECIAL */
	"̤Ƚ",        /* FEEL_TRIED */
#else
	"broken",        /* FEEL_BROKEN */
	"terrible",      /* FEEL_TERRIBLE */
	"worthless",     /* FEEL_WORTHLESS */
	"cursed",        /* FEEL_CURSED */
	"uncursed",      /* FEEL_UNCURSED */
	"average",       /* FEEL_AVERAGE */
	"good",          /* FEEL_GOOD */
	"excellent",     /* FEEL_EXCELLENT */
	"special",       /* FEEL_SPECIAL */
	"tried",         /* FEEL_TRIED */
#endif

};

cptr skill_lev_str[MAX_SKILL_LEVEL] =
#ifdef JP
{"[鿴]", "[]", "[]", "[]", "[ѡ]", "[ã]"};
#else
{"[Beginner]", "[Novice]", "[Average]", "[Skilled]", "[Expert]", "[Master]"};
#endif

s16b skill_lev_var[51] =
{
	1, /*  0 */
	2, /*  1 */
	2, /*  2 */
	2, /*  3 */
	2, /*  4 */
	2, /*  5 */
	2, /*  6 */
	2, /*  7 */
	3, /*  8 */
	3, /*  9 */
	3, /* 10 */
	3, /* 11 */
	3, /* 12 */
	3, /* 13 */
	3, /* 14 */
	3, /* 15 */
	3, /* 16 */
	3, /* 17 */
	3, /* 18 */
	3, /* 19 */
	3, /* 20 */
	4, /* 21 */
	4, /* 22 */
	4, /* 23 */
	4, /* 24 */
	4, /* 25 */
	4, /* 26 */
	4, /* 27 */
	4, /* 28 */
	5, /* 29 */
	5, /* 30 */
	5, /* 31 */
	5, /* 32 */
	5, /* 33 */
	5, /* 34 */
	5, /* 35 */
	6, /* 36 */
	6, /* 37 */
	6, /* 38 */
	6, /* 39 */
	6, /* 40 */
	6, /* 41 */
	6, /* 42 */
	6, /* 43 */
	7, /* 44 */
	7, /* 45 */
	7, /* 46 */
	7, /* 47 */
	7, /* 48 */
	7, /* 49 */
	8  /* 50 */
};

byte conv_terrain2feat[MAX_WILDERNESS] =
{
	FEAT_PERM_SOLID,
	FEAT_TOWN,
	FEAT_DEEP_WATER,
	FEAT_SHAL_WATER,
	FEAT_SWAMP,
	FEAT_DIRT,
	FEAT_GRASS,
	FEAT_TREES,
	FEAT_DIRT,
	FEAT_SHAL_LAVA,
	FEAT_DEEP_LAVA,
	FEAT_MOUNTAIN,
	FEAT_TUNDRA,
	FEAT_DEEP_SEA
};

#ifdef JP
cptr silly_attacks[MAX_SILLY_ATTACK] =
{
  "˾ؤ򤫤줿",
  "ʤβ3äƥȸä",
  "ˤФޤ蘆줿",
  "˷ʤᤵ줿",
  "˥ϥʥФ줿",
  "˥ǹ⤵줿",
  "ʤˤ«ǤФ",
  "ʤǥݡ󥰤򤷤",
  "˥١줿",
  "ˡֿιȯű줿",
  "˥áޤ줿",
  "ˤĤäߤ줿",
  "Ϥʤ٤ä",
  "˴ˤ餯񤭤򤵤줿",
  "˼ڶֺѤ򤻤ޤ줿",
  "˥Ȥ᤯줿",
  "Ϥʤμä",
  "夬줿",
  "鰦ι򤵤줿",
  "Ϥʤ500ߤǸۤä",
  "Ϥʤ100̩ˤĤǮä",
  "˥㡼Ĥ",
  "Ϥʤ˵Ĥ",
  "Ϥʤݥꥴ󲽤",
  "˾줿",
  "ϥƥޤμʸ򾧤",
  "ϤʤΥѥ֥å",
  "ϥ饤ɰư",
  "ϾζޥɤϤ˼Ԥ",
  "ϳȻưˤȯͤ",
  "ϥǥ顼ˡ򤷤",
  "˥饤åǹ⤵줿",
  "󽵴ְǥӥǥͤ˸ʤȻ̼򤫤줿",
  "ϥѥץƤ򾧤",
  "ϥѡȥ饮饯ƥޥʥä",
  "ˤ㤬߾åǥϥ줿",
  "˥åȥȥ꡼ॢå򤫤줿",
  "ϤʤĸǤ򤫤ơ123áפȶ",
  "ϡ֤ʤФФФפȤäƶФ",
  "֤󡢥롼٥󥹤γפȸäŤܤĤ",
  "ϸäTOband2仿桪",
};

/* Formatted style for strfmt() */
cptr silly_attacks_other[MAX_SILLY_ATTACK] =
{
  "%s˾ؤ򤫤",
  "%sβ3äƥȸä",
  "%s򤷤Фޤ路",
  "%s˷ʤᤵ",
  "%s˥ϥʥФ",
  "%s򥸥ǹ⤷",
  "%sˤ«ǤФ",
  "%sǥݡ󥰤򤷤",
  "%s˥١",
  "%sˡֿιȯű᤿",
  "%s˥áޤ",
  "%sˤĤäߤ줿",
  "%s٤ä",
  "%sδˤ餯񤭤򤷤",
  "%s˼ڶֺѤ򤻤ޤä",
  "%sΥȤ᤯ä",
  "%sμä",
  "%s夬ä",
  "%s˰ι򤷤",
  "%s500ߤǸۤä",
  "%s100̩ˤĤǮä",
  "˥㡼Ĥ",
  "%s˵Ĥ",
  "%sݥꥴ󲽤",
  "%s򾯤ä",
  "ƥޤμʸ򾧤",
  "%sΥѥ֥å",
  "饤ɰư",
  "ζޥɤϤ˼Ԥ",
  "%s˳Ȼưˤȯͤ",
  "%s˥ǥ顼ˡ򤷤",
  "%s饤åǹ⤷",
  "%s󽵴ְǥӥǥͤ˸ʤȻ̼򤫤",
  "ѥץƤ򾧤",
  "%s˥ѡȥ饮饯ƥޥʥä",
  "%s򤷤㤬߾åǥϥ᤿",
  "%s˥åȥȥ꡼ॢå򤫤",
  "%sĸǤ򤫤ơ123áפȶ",
  "֤ʤФФФפȤäƶФ",
  "֤󡢥롼٥󥹤γפȸäŤܤĤ",
  "äTOband2仿桪",
};
#else

/* Weird melee attack types when hallucinating */
cptr silly_attacks[MAX_SILLY_ATTACK] =
{
	"smothers",
	"hugs",
	"humiliates",
	"whips",
	"kisses",

	"disgusts",
	"pees all over",
	"passes the gas on",
	"makes obscene gestures at",
	"licks",

	"stomps on",
	"swallows",
	"drools on",
	"misses",
	"shrinks",

	"emasculates",
	"evaporates",
	"solidifies",
	"digitizes",
	"insta-kills",

	"massacres",
	"slaughters",
	"drugs",
	"psychoanalyzes",
	"deconstructs",

	"falsifies",
	"disbelieves",
	"molests",
	"pusupusu",
};
#endif

#ifdef JP
cptr elem_names[] =
{
	"",
	"",
	"",
	""
};
#endif

#ifdef JP
cptr E_elem_names[] =
#else
cptr elem_names[] =
#endif
{
	"Fire",
	"Aqua",
	"Earth",
	"Wind"
};

weather_type weather_table[WEATHER_TYPE_NUM][WEATHER_LEVEL_NUM] =
{
	/* Rain */
	{
		{
			"",
			{ 4, -2,  0,  0}
		},
		{
			"ŷ",
			{ 0,  0,  2,  0}
		},
		{
			"",
			{-2,  2,  2,  0}
		},
		{
			"뱫",
			{-8,  4,  0,  0}
		}
	},
	/* Wind */
	{
		{
			"",
			{-4,  0,  4, -8}
		},
		{
			"",
			{ 2,  0,  0,  1}
		},
		{
			"",
			{ 2,  0, -2,  4}
		},
		{
			"˽",
			{ 0,  0, -4,  8}
		}
	},
	/* Temparature */
	{
		{
			"",
			{ 4, -4, -2,  0}
		},
		{
			"뤤",
			{ 2, -2,  2,  0}
		},
		{
			"",
			{-2,  2,  0,  0}
		},
		{
			"˴",
			{-4,  4, -2,  0}
		}
	}
};

cptr ethnicity_names[ETHNICITY_NUM] =
{
#ifdef JP
	"륹",
	"륬",
	"Х",
	"Υӥ",
	"ǥ",
#else
	"Walstanian",
	"Gargastan",
	"Bacrum",
	"Zenobian",
	"Lodis",
#endif
};

special_blow_type special_blow_info[MAX_SB] =
{
	{
		"ӭ컶",
		"󥹥1Τ˸ߤHP򹶷ϤȤ빶Ԥʬ⸽HP1/4򼺤",
		WT_BIT_AXE | WT_BIT_CLAW,
		30, 60, 50, A_STR,
	},
	{
		"Ʋ",
		"󥹥1Τ*б*Ԥ",
		WT_BIT_AXE | WT_BIT_WHIP | WT_BIT_HAMMER,
		35, 45, 55, A_WIS,
	},
	{
		"",
		"󥹥1Τ˹Ԥ",
		WT_BIT_SMALL_SWORD | WT_BIT_SWORD | WT_BIT_KATANA,
		30, 40, 50, A_DEX,
	},
	{
		"ŷ̯",
		"󥹥1Τ**Ԥ",
		WT_BIT_SMALL_SWORD | WT_BIT_SPEAR | WT_BIT_SCYTHE,
		35, 45, 55, A_WIS,
	},
	{
		"ܹ˷",
		"󥹥1Τʬ򹶷Ԥ",
		WT_BIT_BOW,
		30, 40, 50, A_STR,
	},
	{
		"ε",
		"󥹥1Τ˹ԤƤˤäƶѤ롣",
		WT_BIT_SWORD | WT_BIT_GREAT_SWORD | WT_BIT_KATANA,
		45, 75, 70, A_STR,
	},
	{
		"",
		"󥹥1ΤвԤвʤɬв롣",
		WT_BIT_SPEAR | WT_BIT_HAMMER,
		30, 45, 70, A_STR,
	},
	{
		"ŷ˻",
		"󥹥1Τ˹ԤΥåХå롣",
		WT_BIT_KATANA | WT_BIT_STAFF | WT_BIT_HAMMER,
		30, 40, 50, A_CON,
	},
	{
		"ư",
		"󥹥1Τ̿Ϥ빶Ԥ",
		WT_BIT_AXE | WT_BIT_SPEAR | WT_BIT_CLAW,
		40, 50, 60, A_INT,
	},
	{
		"ٲ",
		"󥹥1Τ˸ߤμʬμƤ᡼Ʊ᡼Ϳ롣Ϳ᡼Ϥ롣",
		WT_BIT_GREAT_SWORD | WT_BIT_LANCE | WT_BIT_SCYTHE,
		45, 85, 75, A_INT,
	},
};

special_blow_type temple_blow_info[MAX_TEMPLE_SB] =
{
	{
		"֥åץꥺ",
		"󥹥1Τ˰ŹԤ㤵롣",
		WT_BIT_AXE | WT_BIT_CLAW | WT_BIT_WHIP,
		30, 45, 50, A_STR,
	},
	{
		"ǡ",
		"󥹥1Τ*б*Ԥ̥λ롣",
		WT_BIT_WHIP | WT_BIT_SCYTHE,
		35, 45, 55, A_CHR,
	},
	{
		"ǥ٥󥸥㡼",
		"󥹥1Τ**ԤΥåХå롣",
		WT_BIT_GREAT_SWORD | WT_BIT_AXE | WT_BIT_HAMMER,
		35, 50, 55, A_CON,
	},
	{
		"եΥ",
		"󥹥1Τ˿并Ԥ",
		WT_BIT_SWORD | WT_BIT_SPEAR,
		30, 45, 50, A_DEX,
	},
	{
		"֥쥤",
		"󥹥1Τŷ⹶Ԥ",
		WT_BIT_CLAW | WT_BIT_SPEAR,
		25, 40, 40, A_STR,
	},
	{
		"ե쥤ߥ󥰥ǥ",
		"󥹥1Τ*б*Ԥ",
		WT_BIT_KATANA | WT_BIT_SPEAR | WT_BIT_STAFF,
		30, 45, 55, A_WIS,
	},
	{
		"饤åȥС",
		"󥹥1Τʤ빶ԤˡǤʤǥåɤʤгμ¤˾Ǥ롣",
		WT_BIT_SMALL_SWORD | WT_BIT_STAFF,
		40, 65, 70, A_WIS,
	},
	{
		"ݥץ",
		"󥹥1Τ̿Ϥ빶Ԥ",
		WT_BIT_SWORD | WT_BIT_LANCE | WT_BIT_SCYTHE,
		40, 50, 60, A_INT,
	},
};


tarot_type tarot_info[] =
{
#ifdef JP
	{
		"",
		"ϻȤȥåȥɤϤ롣ˤϼ",
		50000,
		TRUE,
	},
	{
		"ѻա",
		"ϻȤǽκͤ夬롣",
		300000,
		FALSE,
	},
	{
		"ѻադεհ",
		"ϻȤǽκͤ롣",
		0,
		FALSE,
	},
	{
		"Խġ",
		"ϻȤϤ˲иͤơܡǡСۯ۰ڤβ˿ʹԤв",
		25000,
		TRUE,
	},
	{
		"Խġդεհ",
		"ϻȤϤȾиͤơꡢܡǡСۯ۰ڤ˿ʹԤвˤ롣",
		0,
		FALSE,
	},
	{
		"Խ",
		"ϻȤ̥Ϥκͤ夬롣",
		300000,
		FALSE,
	},
	{
		"Խդεհ",
		"ϻȤ̥Ϥκͤ롣",
		0,
		FALSE,
	},
	{
		"Թ",
		"ϻȤȰŪ˹롣",
		50000,
		TRUE,
	},
	{
		"Թդεհ",
		"ϻȤȰŪ˹롣",
		0,
		FALSE,
	},
	{
		"ˡġ",
		"ϻȤȤ̱λٻ롣",
		50000,
		FALSE,
	},
	{
		"ˡġդεհ",
		"ϻȤȤ̱λٻ򼺤",
		0,
		FALSE,
	},
	{
		"͡",
		"ϻȤȻ볦Υ󥹥Ϥ̥λ롣",
		85000,
		TRUE,
	},
	{
		"͡դεհ",
		"ϻȤȥեƤΥ󥹥ܳФᤵ롣",
		0,
		FALSE,
	},
	{
		"֡",
		"ϻȤȵǽʥ󥹥1ΤڥåȤȤƾ롣",
		50000,
		TRUE,
	},
	{
		"֡դεհ",
		"ϻȤȵǽʥ󥹥1ΤŨȤƾ롣",
		0,
		FALSE,
	},
	{
		"ϡ",
		"ϻȤϤκͤ夬롣",
		300000,
		FALSE,
	},
	{
		"ϡդεհ",
		"ϻȤϤκͤ롣",
		0,
		FALSE,
	},
	{
		"Աԡ",
		"ϻȤȸκͤ夬롣",
		300000,
		FALSE,
	},
	{
		"Աԡդεհ",
		"ϻȤȸκͤ롣",
		0,
		FALSE,
	},
	{
		"Ա̿ء",
		"ϻȤȸߤγƹ롣",
		25000,
		TRUE,
	},
	{
		"Ա̿ءդεհ",
		"ϻȤȼʬѿȤ褦Ȥ롣",
		5000,
		FALSE,
	},
	{
		"",
		"ϻȤȥ饤Ȥ˷",
		50000,
		TRUE,
	},
	{
		"դεհ",
		"ϻȤȥ饤Ȥ٤˷",
		50000,
		TRUE,
	},
	{
		"ߤ줿ˡ",
		"ϻȤŤαǰȯư롣",
		0,
		FALSE,
	},
	{
		"ߤ줿ˡդεհ",
		"ϻȤȼʬΥԡɤʵפ1롣",
		0,
		FALSE,
	},
	{
		"Ի",
		"ϻȤϤͤ20%ޤǸϤκͤȸͤκʬ᡼ȤƥեƤΥ󥹥Ϳ롣",
		25000,
		FALSE,
	},
	{
		"Իդεհ",
		"ϻȤƤǽϴ͡ϡMPǽ򤤤餫γǰѤޤ޼ʬκ٥1᤹",
		250000,
		TRUE,
	},
	{
		"",
		"ϻȤȸߤηиͤȾʬ(100000Ķʤ)ηи롣",
		25000,
		FALSE,
	},
	{
		"դεհ",
		"ϻȤȸߤηиͤȾʬ(100000Ķʤ)ηиκͤ򼺤",
		0,
		FALSE,
	},
	{
		"԰",
		"ϻȤȥǡ1ΤŨȤƾ롣˥ڥåȤˤʤ롣",
		0,
		FALSE,
	},
	{
		"԰դεհ",
		"ϻȤȻ볦Υǡ300Υ᡼Ϳ롣",
		25000,
		TRUE,
	},
	{
		"",
		"ϻȤȶˤƹϰϤ˲250Υ᡼롣",
		25000,
		FALSE,
	},
	{
		"դεհ",
		"ϻȤȼդΥƥࡢ󥹥Ϸ˲롣",
		5000,
		FALSE,
	},
	{
		"",
		"ϻȤʣιʥƥФ",
		200000,
		FALSE,
	},
	{
		"դεհ",
		"ϻȤӤ()Ѥ롣",
		0,
		FALSE,
	},
	{
		"Է",
		"ϻȤȼʬΥ줿ʣΥ󥹥򾤴롣",
		0,
		FALSE,
	},
	{
		"Էդεհ",
		"ϻȤȼʬΥ줿Υ󥹥ʣä롣ˡ󥹥ˤ̵",
		50000,
		TRUE,
	},
	{
		"ۡ",
		"ϻȤȤγϷΤ뤳ȤǤγΤʵפ˾Ȥ餵졢γƤΥ󥹥櫡⡢ʡƥƥΤ롣",
		8000,
		TRUE,
	},
	{
		"ۡդεհ",
		"ϻȤȤγΤŤ롣",
		0,
		FALSE,
	},
	{
		"ԿȽ",
		"ϻȤɤΥ󥹥ȥڥåȰʳƤΥ󥹥ä롣ˡ󥹥ˤ̵",
		300000,
		TRUE,
	},
	{
		"ԿȽդεհ",
		"ϻȤɤΥ󥹥ʣŨȤƾ롣",
		0,
		FALSE,
	},
	{
		"",
		"ϻȤȥƥ1ĴꤦȤǤ롣",
		5000000,
		TRUE,
	},
	{
		"դεհ",
		"ϻȤ1Υƥä롣",
		0,
		FALSE,
	},
	{
		"Զԡ",
		"ϻȤ⤭ʤꡢˡμΨ夬롣γˤ¤̤³",
		0,
		FALSE,
	},
	{
		"Զԡդεհ",
		"ϻȤƤΥ󥹥⤭ʤ롣γˤ¤̤³",
		100000,
		TRUE,
	},
#else
	{
		"the Blank Card",
		"",
		50000,
		TRUE,
	},
	{
		"the Magician",
		"",
		300000,
		FALSE,
	},
	{
		"reverted position of the Magician",
		"",
		0,
		FALSE,
	},
	{
		"the High Priestess",
		"",
		25000,
		TRUE,
	},
	{
		"reverted position of the High Priestess",
		"",
		0,
		FALSE,
	},
	{
		"the Empress",
		"",
		300000,
		FALSE,
	},
	{
		"reverted position of the Empress",
		"",
		0,
		FALSE,
	},
	{
		"the Emperor",
		"",
		50000,
		TRUE,
	},
	{
		"reverted position of the Emperor",
		"",
		0,
		FALSE,
	},
	{
		"the Hierophant",
		"",
		50000,
		FALSE,
	},
	{
		"reverted position of the Hierophant",
		"",
		0,
		FALSE,
	},
	{
		"the Lovers",
		"",
		85000,
		TRUE,
	},
	{
		"reverted position of the Lovers",
		"",
		0,
		FALSE,
	},
	{
		"the Chariot",
		"",
		50000,
		TRUE,
	},
	{
		"reverted position of the Chariot",
		"",
		0,
		FALSE,
	},
	{
		"the Strength",
		"",
		300000,
		FALSE,
	},
	{
		"reverted position of the Strength",
		"",
		0,
		FALSE,
	},
	{
		"the Hermit",
		"",
		300000,
		FALSE,
	},
	{
		"reverted position of the Hermit",
		"",
		0,
		FALSE,
	},
	{
		"the Wheel of Fortune",
		"",
		25000,
		TRUE,
	},
	{
		"reverted position of the Wheel of Fortune",
		"",
		5000,
		FALSE,
	},
	{
		"the Justice",
		"",
		50000,
		TRUE,
	},
	{
		"reverted position of the Justice",
		"",
		50000,
		TRUE,
	},
	{
		"the Hanged Man",
		"",
		0,
		FALSE,
	},
	{
		"reverted position of the Hanged Man",
		"",
		0,
		FALSE,
	},
	{
		"the Death",
		"",
		25000,
		FALSE,
	},
	{
		"reverted position of the Death",
		"",
		250000,
		TRUE,
	},
	{
		"the Temperance",
		"",
		25000,
		FALSE,
	},
	{
		"reverted position of the Temperance",
		"",
		0,
		FALSE,
	},
	{
		"the Devil",
		"",
		0,
		FALSE,
	},
	{
		"reverted position of the Devil",
		"",
		25000,
		TRUE,
	},
	{
		"the Tower",
		"",
		25000,
		FALSE,
	},
	{
		"reverted position of the Tower",
		"",
		5000,
		FALSE,
	},
	{
		"the Star",
		"",
		200000,
		FALSE,
	},
	{
		"reverted position of the Star",
		"",
		0,
		FALSE,
	},
	{
		"the Moon",
		"",
		0,
		FALSE,
	},
	{
		"reverted position of the Moon",
		"",
		50000,
		TRUE,
	},
	{
		"the Sun",
		"",
		8000,
		TRUE,
	},
	{
		"reverted position of the Sun",
		"",
		0,
		FALSE,
	},
	{
		"the Judgement",
		"",
		300000,
		TRUE,
	},
	{
		"reverted position of the Judgement",
		"",
		0,
		FALSE,
	},
	{
		"the World",
		"",
		5000000,
		TRUE,
	},
	{
		"reverted position of the World",
		"",
		0,
		FALSE,
	},
	{
		"the Fool",
		"",
		0,
		FALSE,
	},
	{
		"reverted position of the Fool",
		"",
		100000,
		TRUE,
	},
#endif
};

scratch_card_type scratch_card_info[] =
{
	{
		1,
		0,
		0,
		"ϥ",
	},
	{
		10000000L,
		0, /* Hack */
		0, /* Hack */
		"ʤϤä̴˰㤤ʤ",
	},
	{
		4000000L,
		TV_LITE,
		SV_LITE_MAGICAL_LAMP,
		"ޤޤ̴Τ褦ɤݤ",
	},
	{
		1000000L,
		TV_SCROLL,
		SV_SCROLL_SNAP_DRAGON,
		"äۤä",
	},
	{
		800000L,
		TV_STAFF,
		SV_STAFF_GENOCIDE,
		"ä",
	},
	{
		500000L,
		TV_SCROLL,
		SV_SCROLL_ARTIFACT,
		"ä",
	},
	{
		200000L,
		TV_ROD,
		SV_ROD_IDENTIFY,
		"ϻä",
	},
	{
		100000L,
		TV_ROD,
		SV_ROD_SPEED,
		"ä",
	},
	{
		80000L,
		TV_WAND,
		SV_WAND_DRAGON_BREATH,
		"Ȭä",
	},
	{
		40000L,
		TV_STAFF,
		SV_STAFF_HEALING,
		"ä",
	},
	{
		20000,
		TV_AMULET,
		0, /* Hack */
		"ä",
	},
	{
		10000,
		TV_AMULET,
		SV_AMULET_RESISTANCE,
		"ä",
	},
	{
		5000,
		TV_SCROLL,
		SV_SCROLL_DEADLY_BLOW_LEARNING,
		"ä",
	},
	{
		2000,
		TV_SCROLL,
		SV_SCROLL_EGO_CREATION,
		"ä",
	},
	{
		1000,
		TV_STONE,
		0,
		"ä",
	},
	{
		500,
		TV_SCROLL,
		SV_SCROLL_STAR_REMOVE_CURSE,
		"ä",
	},
	{
		200,
		TV_CLOAK,
		SV_RAINCOAT,
		"ϻä",
	},
	{
		32,
		TV_SCROLL,
		SV_SCROLL_WORD_OF_RECALL,
		"",
	},
	{
		8,
		TV_FOOD,
		SV_FOOD_PINT_OF_WINE,
		"Ȭ",
	},
	{
		4,
		TV_FOOD,
		SV_FOOD_PINT_OF_ALE,
		"",
	},
	{
		2,
		TV_FOOD,
		SV_FOOD_BISCUIT,
		"",
	},
};

int a_to_tr[A_MAX] =
{
	TR_STR,
	TR_INT,
	TR_WIS,
	TR_DEX,
	TR_CON,
	TR_CHR,
};

int ob_to_tr[OB_MAX] =
{
	TR_MAGIC_MASTERY,
	TR_STEALTH,
	TR_SEARCH,
	TR_INFRA,
	TR_TUNNEL,
	TR_SPEED,
	TR_BLOWS,
	TR_ANTI_MAGIC,
};

int ali_to_tr[ALI_MAX] =
{
	TR_ALIGN_LNC,
	TR_ALIGN_GNE,
};

cptr weapon_skill_name[MAX_WT] =
{
#ifdef JP
	"ʤ",
	"ͷ",
	"",
	"",
	"",
	"",
	"",
	"",
	"",
	"",
	"",
	"ϥޡ",
	"",
	"",
	"",
	"",
#else
	"none",
	"Small sword",
	"Katana",
	"Sword",
	"Great sword",
	"Axe",
	"Spear",
	"Lance",
	"Craw",
	"Scythe",
	"Whip",
	"Hammer",
	"Staff",
	"Fan",
	"Bow",
	"Gun",
#endif
};

/*
 * The table of "symbol info" -- each entry is a string of the form
 * "X:desc" where "X" is the trigger, and "desc" is the "info".
 */
cptr ident_info[] =
{
#ifdef JP
	" :Ű",
	"!:, ",
	"\":ߥå, ",
	"#:(ɥ)/ʪ/",
	"$:(⤫)",
	"%:̮(ϴ䤫б)",
	"&:Ȣ",
	"':ɥ",
	"(:餫ɶ",
	"):",
	"*:ޤ̮ޤϵβʪ",
	"+:Ĥɥ",
	",:ʪ, ФΥ",
	"-:ˡ, å",
	".:",
	"/:Ⱦ(å/ѥ/)",
	"0:ʪۤ",
	"1:߲",
	"2:ɶ񲰤",
	"3:Ź",
	"4:",
	"5:ϣѤŹ",
	"6:ˡŹ",
	"7:֥åޡåȤ",
	"8:椬Ȥ",
	"9:Ź",
	"::",
	";:Ħ/ȯΥ롼",
	"<:곬",
	"=:",
	">:곬",
	"?:ʪ",
	"@:ץ쥤䡼",
	"A:ŷ",
	"B:Ļ",
	"C:",
	"D:ɥ饴/磻",
	"E:󥿥",
	"F:ͭΰ",
	"G:",
	"H:",
	"I:",
	"J:إ",
	"K:顼ӡȥ",
	"L:å",
	"M:¿",
	/* "N:unused", */
	"O:",
	"P:ʹַʪ",
	"Q:륹륰(̮Ǥ)",
	"R:/ξ",
	"S://",
	"T:ȥ",
	"U:ǡ",
	"V:ѥ",
	"W:磻/쥤/",
	"X://",
	"Y:ƥ",
	"Z:ϥ",
	"[:ޡ",
	"\\:ߴ(ᥤ//)",
	"]:ɶ",
	"^:ȥå",
	"_:",
	"`:ͷĦ",
	"a:",
	"b:",
	"c:५",
	"d:ɥ饴",
	"e:ܶ",
	"f:ͥ",
	"g:",
	"h:/ۥӥå/ꥶɥޥ",
	"i:٥ȥ٥",
	"j:꡼",
	"k:ܥ",
	"l:ʪ",
	"m:",
	"n:ʡ",
	"o:",
	"p:ʹ",
	"q:­",
	"r:ͥ",
	"s:ȥ",
	"t:Įο",
	"u:ǡ",
	"v:ܥƥå",
	"w:ॷ/緲",
	/* "x:unused", */
	"y:",
	"z:/ߥ",
	"{:ƻ(/)",
	"|:(//)",
	"}:ƻ(/ܥ/)",
	"~:/ϴή(Υƥ)",
#else
	" :A dark grid",
	"!:A potion (or oil)",
	"\":An amulet (or necklace)",
	"#:A wall (, secret door or plant)",
	"$:Treasure (gold or gems)",
	"%:A vein (magma or quartz)",
	"&:A chest",
	"':An open door",
	"(:Soft armor",
	"):A shield",
	"*:A vein with treasure or a ball monster",
	"+:A closed door",
	",:Food (or mushroom patch)",
	"-:A wand (or rod)",
	".:Floor",
	"/:A polearm (Axe/Pike/etc)",
	"0:Entrance to Museum",
	"1:Entrance to General Store",
	"2:Entrance to Armory",
	"3:Entrance to Weaponsmith",
	"4:Entrance to Temple",
	"5:Entrance to Alchemy shop",
	"6:Entrance to Magic store",
	"7:Entrance to Black Market",
	"8:Entrance to your home",
	"9:Entrance to the bookstore",
	"::Rubble",
	";:A glyph of warding / an explosive rune",
	"<:An up staircase",
	"=:A ring",
	">:A down staircase",
	"?:A scroll",
	"@:You",
	"A:Angel",
	"B:Bird",
	"C:Canine",
	"D:Ancient Dragon/Wyrm",
	"E:Elemental",
	"F:Winged Demihuman",
	"G:Ghost",
	"H:Hybrid",
	"I:Insect",
	"J:Snake",
	"K:Killer Beetle",
	"L:Lich",
	"M:Multi-Headed Reptile",
	/* "N:unused", */
	"O:Ogre",
	"P:Giant Humanoid",
	"Q:Quylthulg (Pulsing Flesh Mound)",
	"R:Reptile/Amphibian",
	"S:Spider/Scorpion/Tick",
	"T:Troll",
	"U:Major Demon",
	"V:Vampire",
	"W:Wight/Wraith/etc",
	"X:Xorn/Xaren/etc",
	"Y:Yeti",
	"Z:Zephyr Hound",
	"[:Hard armor",
	"\\:A hafted weapon (mace/whip/etc)",
	"]:Misc. armor",
	"^:A trap",
	"_:A staff",
	"`:A figurine or statue",
	"a:Ant",
	"b:Bat",
	"c:Centipede",
	"d:Dragon",
	"e:Floating Eye",
	"f:Feline",
	"g:Golem",
	"h:Hobbit/Elf/Dwarf",
	"i:Icky Thing",
	"j:Jelly",
	"k:Kobold",
	"l:Aquatic monster",
	"m:Mold",
	"n:Naga",
	"o:Orc",
	"p:Person/Human",
	"q:Quadruped",
	"r:Rodent",
	"s:Skeleton",
	"t:Townsperson",
	"u:Minor Demon",
	"v:Vortex",
	"w:Worm/Worm-Mass",
	/* "x:unused", */
	"y:Yeek",
	"z:Zombie/Mummy",
	"{:A missile (arrow/bolt/shot)",
	"|:An edged weapon (sword/dagger/etc)",
	"}:A launcher (bow/crossbow/gun)",
	"~:Fluid terrain (or miscellaneous item)",
#endif

	NULL
};


